import { DefineValuePlaceholder, GLSLContext, KernelBundle } from '@antv/g-webgpu-core';
import { Program as ComputeProgram } from './ast/glsl-tree';
import { Program as TSProgram } from './ast/ts-estree';
import { ICodeGenerator, Target } from './backends/ICodeGenerator';
export { DefineValuePlaceholder };
export interface ProgramParams {
    dispatch: [number, number, number];
    maxIteration: number;
    bindings: Array<{
        name: string;
        data: number | number[];
    }>;
}
export declare const FunctionPrependPlaceholder = "__FunctionPrependPlaceholder__";
export declare class Compiler {
    private target;
    private glslContext;
    private generators;
    private transformer;
    setTarget(target: Target): void;
    getContext(): GLSLContext;
    getGenerator(): ICodeGenerator;
    /**
     * 生成 AST
     */
    parse(source: string): TSProgram | undefined;
    /**
     * AST 变换
     * ESTree -> ShaderTree
     */
    transform(program: TSProgram): ComputeProgram;
    /**
     * 代码生成
     */
    generateCode(program: ComputeProgram): string;
    clear(): void;
    compileBundle(gCode: string, targets?: Target[]): KernelBundle;
}
