import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import { DefineValuePlaceholder } from '@antv/g-webgpu-core';
import { CodeGeneratorGLSL100 } from './backends/CodeGeneratorGLSL100';
import { CodeGeneratorGLSL450 } from './backends/CodeGeneratorGLSL450';
import { CodeGeneratorWGSL } from './backends/CodeGeneratorWGSL';
import { Target } from './backends/ICodeGenerator';
import { parse as _parse } from './pegjs/g';
import { Transformer } from './Transformer';
// export interface Program {
//   type: 'Program';
//   body: Node[];
// }
export { DefineValuePlaceholder };
export var FunctionPrependPlaceholder = '__FunctionPrependPlaceholder__';
export var Compiler = /*#__PURE__*/function () {
  function Compiler() {
    var _this$generators;

    _classCallCheck(this, Compiler);

    this.target = Target.GLSL100;
    this.glslContext = {
      name: '',
      dispatch: [1, 1, 1],
      threadGroupSize: [1, 1, 1],
      maxIteration: 1,
      defines: [],
      uniforms: [],
      globalDeclarations: [],
      output: {
        name: ''
      },
      needPingpong: false
    };
    this.generators = (_this$generators = {}, _defineProperty(_this$generators, Target.GLSL100, new CodeGeneratorGLSL100()), _defineProperty(_this$generators, Target.GLSL450, new CodeGeneratorGLSL450()), _defineProperty(_this$generators, Target.WGSL, new CodeGeneratorWGSL()), _this$generators);
    this.transformer = new Transformer();
  }

  _createClass(Compiler, [{
    key: "setTarget",
    value: function setTarget(target) {
      this.target = target;
      this.transformer.target = target;
      this.clear();
    }
  }, {
    key: "getContext",
    value: function getContext() {
      return this.glslContext;
    }
  }, {
    key: "getGenerator",
    value: function getGenerator() {
      return this.generators[this.target];
    }
    /**
     * 生成 AST
     */

  }, {
    key: "parse",
    value: function parse(source) {
      try {
        return _parse(source);
      } catch (e) {
        // tslint:disable-next-line:no-console
        console.error('[Parse error]:', e);
        throw e;
      }
    }
    /**
     * AST 变换
     * ESTree -> ShaderTree
     */

  }, {
    key: "transform",
    value: function transform(program) {
      return this.transformer.transform(program, this.glslContext);
    }
    /**
     * 代码生成
     */

  }, {
    key: "generateCode",
    value: function generateCode(program) {
      var generator = this.getGenerator();
      generator.clear();
      return generator.generate(program, this.glslContext);
    }
  }, {
    key: "clear",
    value: function clear() {
      this.glslContext = {
        name: '',
        dispatch: [1, 1, 1],
        threadGroupSize: [1, 1, 1],
        maxIteration: 1,
        defines: [],
        uniforms: [],
        globalDeclarations: [],
        output: {
          name: ''
        },
        needPingpong: false
      };
    }
  }, {
    key: "compileBundle",
    value: function compileBundle(gCode) {
      var _shaders,
          _this = this;

      var targets = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [Target.WGSL, Target.GLSL450, Target.GLSL100];
      var bundle = {
        shaders: (_shaders = {}, _defineProperty(_shaders, Target.WGSL, ''), _defineProperty(_shaders, Target.GLSL450, ''), _defineProperty(_shaders, Target.GLSL100, ''), _shaders),
        context: undefined,
        toString: function toString() {
          return JSON.stringify(bundle).replace(/\\/g, '\\\\');
        }
      };
      targets.forEach(function (target) {
        _this.setTarget(target);

        var estree = _this.parse(gCode); // TODO: 应该只需要 transform 一次


        var shaderTree = _this.transform(estree);

        bundle.shaders[target] = _this.generateCode(shaderTree);
      }); // @ts-ignore

      bundle.context = this.getContext();
      return bundle;
    }
  }]);

  return Compiler;
}();
//# sourceMappingURL=Compiler.js.map