import { GLSLContext } from '@antv/g-webgpu-core';
import { ArrayExpression, AssignmentExpression, BinaryExpression, BlockStatement, CallExpression, ConditionalExpression, DataType, DoWhileStatement, Expression, ForStatement, FunctionDeclaration, FunctionExpression, IfStatement, MemberExpression, NumThreadStatement, Program, ReturnStatement, Statement, UnaryExpression, UpdateExpression, VariableDeclaration, WhileStatement } from '../ast/glsl-tree';
import { ICodeGenerator } from './ICodeGenerator';
export declare class CodeGeneratorWGSL implements ICodeGenerator {
    static GWEBGPU_UNIFORM_PARAMS: string;
    static GWEBGPU_BUFFER: string;
    private context;
    private bufferBindingIndex;
    clear(): void;
    generate(program: Program, context?: GLSLContext): string;
    generateStatement(stmt: Statement): string;
    generateNumThreadStatement(stmt: NumThreadStatement): string;
    generateVariableDeclaration(stmt: VariableDeclaration): string;
    generateBlockStatement(stmt: BlockStatement): string;
    generateReturnStatement(stmt: ReturnStatement): string;
    generateWhileStatement(stmt: WhileStatement): string;
    generateDoWhileStatement(stmt: DoWhileStatement): string;
    generateForStatement(node: ForStatement): string;
    generateIfStatement(node: IfStatement): string;
    generateFunctionExpression(stmt: FunctionDeclaration | FunctionExpression): string;
    generateExpression(expression: Expression | null, dataType?: DataType): string;
    generateUnaryExpression(expression: UnaryExpression, dataType?: DataType): string;
    generateConditionalExpression(expression: ConditionalExpression): string;
    generateMemberExpression(expression: MemberExpression): string;
    generateUpdateExpression(expression: UpdateExpression): string;
    generateAssignmentExpression(expression: AssignmentExpression, dataType?: DataType): string;
    generateArrayExpression(expression: ArrayExpression, dataType?: DataType): string;
    generateBinaryExpression(expression: BinaryExpression, dataType?: DataType): string;
    generateCallExpression(expression: CallExpression, dataType?: DataType): string;
    generateDataType(dt: DataType): string;
    generateScalar(dt: DataType, scalar: number | boolean): string;
    generateVector(dt: DataType, vector: number | boolean): string;
    generateMatrix(dt: DataType, vector: number | boolean): string;
    private wrapFloat;
    private castingDataType;
    private generateSwizzling;
    private generateGlobalVariableDeclarations;
    private generateUniforms;
    private generateBuffers;
}
