import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _defineProperty from "@babel/runtime/helpers/defineProperty";

var _dataTypeMap;

import { DefineValuePlaceholder } from '@antv/g-webgpu-core';
import { AST_NODE_TYPES, AST_TOKEN_TYPES, STORAGE_CLASS } from '../ast/glsl-ast-node-types';
import { dataTypeMap as transformDataTypeMap } from '../Transformer';
import { componentWiseFunctions, importFunctions, typeCastFunctions } from '../utils/builtin-functions';
import { compareDataTypePriority, getByteCount, getComponentDataTypeFromVector, getComponentSize, isArray, isMatrix, isScalar, isVector } from '../utils/data-type';
import { getIdentifierFromScope, traverseUpwards } from '../utils/node';
import { Target } from './ICodeGenerator';
var dataTypeMap = (_dataTypeMap = {}, _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Void, 'void'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Boolean, 'bool'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Int32, 'i32'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Uint32, 'u32'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Float, 'f32'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector2Float, 'vec2<f32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector3Float, 'vec3<f32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector4Float, 'vec4<f32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector2Int, 'vec2<i32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector3Int, 'vec3<i32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector4Int, 'vec4<i32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector2Uint, 'vec2<u32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector3Uint, 'vec3<u32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector4Uint, 'vec4<u32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector2Boolean, 'vec2<bool>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector3Boolean, 'vec3<bool>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector4Boolean, 'vec4<bool>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Matrix3x3Float, 'mat3x3<f32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Matrix4x4Float, 'mat4x4<f32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.FloatArray, 'array<f32>'), _defineProperty(_dataTypeMap, AST_TOKEN_TYPES.Vector4FloatArray, 'array<vec4<f32>>'), _dataTypeMap);
export var CodeGeneratorWGSL = /*#__PURE__*/function () {
  function CodeGeneratorWGSL() {
    _classCallCheck(this, CodeGeneratorWGSL);

    this.context = void 0;
    this.bufferBindingIndex = -1;
  }

  _createClass(CodeGeneratorWGSL, [{
    key: "clear",
    value: function clear() {
      this.bufferBindingIndex = -1;
    }
  }, {
    key: "generate",
    value: function generate(program, context) {
      var _this = this;

      if (context) {
        this.context = context;
      } // @see https://gpuweb.github.io/gpuweb/wgsl.html#builtin-variables


      var body = program.body.map(function (stmt) {
        return _this.generateStatement(stmt);
      }).join('\n');
      return "import \"GLSL.std.450\" as std;\n".concat(importFunctions[Target.WGSL], "\n\n# var gWebGPUDebug : bool = false;\n# var gWebGPUDebugOutput : vec4<f32> = vec4<f32>(0.0);\n\n[[builtin global_invocation_id]] var<in> globalInvocationID : vec3<u32>;\n# [[builtin work_group_size]] var<in> workGroupSize : vec3<u32>;\n# [[builtin work_group_id]] var<in> workGroupID : vec3<u32>;\n[[builtin local_invocation_id]] var<in> localInvocationID : vec3<u32>;\n# [[builtin num_work_groups]] var<in> numWorkGroups : vec3<u32>;\n[[builtin local_invocation_idx]] var<in> localInvocationIndex : u32;\n\n").concat(this.generateUniforms(), "\n").concat(this.generateBuffers(), "\n").concat(this.generateGlobalVariableDeclarations(), "\n\n").concat(body, "\n\nentry_point compute as \"main\" = main;\n");
    }
  }, {
    key: "generateStatement",
    value: function generateStatement(stmt) {
      if (stmt.type === AST_NODE_TYPES.VariableDeclaration) {
        return this.generateVariableDeclaration(stmt);
      } else if (stmt.type === AST_NODE_TYPES.FunctionDeclaration) {
        return this.generateFunctionExpression(stmt);
      } else if (stmt.type === AST_NODE_TYPES.BlockStatement) {
        return this.generateBlockStatement(stmt);
      } else if (stmt.type === AST_NODE_TYPES.NumThreadStatement) {
        return this.generateNumThreadStatement(stmt);
      } else {
        // tslint:disable-next-line:no-console
        console.warn("[WGSL compiler]: unknown statement: ".concat(stmt.type));
      }

      return '';
    }
  }, {
    key: "generateNumThreadStatement",
    value: function generateNumThreadStatement(stmt) {
      // 需要作为 main 函数属性声明
      return '';
    }
  }, {
    key: "generateVariableDeclaration",
    value: function generateVariableDeclaration(stmt) {
      var _this2 = this;

      return stmt.declarations.map(function (declarator) {
        if (declarator.storageClass === STORAGE_CLASS.UniformConstant) {// @see https://gpuweb.github.io/gpuweb/wgsl.html#module-constants
          // eg. const golden : f32 = 1.61803398875;
          // const define = this.context?.defines.find(
          //   (d) => d.name === declarator.id.name,
          // );
          // const placeHolder = `${DefineValuePlaceholder}${define?.name}`;
          // return `const ${declarator.id.name} : ${this.generateDataType(
          //   declarator.id.dataType,
          // )} = ${
          //   define?.runtime
          //     ? placeHolder
          //     : this.generateExpression(declarator.init)
          // };`;
          // TODO: v-0022 Global variables must have a storage class
          // https://gpuweb.github.io/gpuweb/wgsl.html#validation
        } else if (declarator.storageClass === STORAGE_CLASS.Uniform || declarator.storageClass === STORAGE_CLASS.StorageBuffer) {// const name = declarator.id.name;
          // const componentDataType = getComponentDataTypeFromVector(
          //   declarator.id.dataType,
          // );
          // const componentDataTypeStr = this.generateDataType(componentDataType);
          // const dataType = this.generateDataType(declarator.id.dataType);
          // 不在这里生成
        } else if (declarator.storageClass === STORAGE_CLASS.Private) {
          if (declarator.init) {
            // var a : f32 = 1.0;
            // var b : vec3<f32> = vec3<f32>(1.0, 1.0, 1.0);
            return "var ".concat(declarator.id.name, " : ").concat(_this2.generateDataType(declarator.id.dataType), " = ").concat(_this2.generateExpression(declarator.init, declarator.id.dataType), ";");
          } else {
            // var a : f32;
            // Root Scope 中不允许声明变量但不赋值
            if (stmt.parent && stmt.parent.type === AST_NODE_TYPES.Program) {
              return '';
            }

            return "var ".concat(declarator.id.name, " : ").concat(_this2.generateDataType(declarator.id.dataType), ";");
          }
        }

        return '';
      }).join('\n');
    }
  }, {
    key: "generateBlockStatement",
    value: function generateBlockStatement(stmt) {
      var _this3 = this;

      return stmt.body.map(function (s) {
        if (s.type === AST_NODE_TYPES.VariableDeclaration) {
          return _this3.generateVariableDeclaration(s);
        } else if (s.type === AST_NODE_TYPES.ExpressionStatement) {
          return _this3.generateExpression(s.expression) + ';';
        } else if (s.type === AST_NODE_TYPES.ReturnStatement) {
          return _this3.generateReturnStatement(s);
        } else if (s.type === AST_NODE_TYPES.IfStatement) {
          return _this3.generateIfStatement(s);
        } else if (s.type === AST_NODE_TYPES.ForStatement) {
          return _this3.generateForStatement(s);
        } else if (s.type === AST_NODE_TYPES.BreakStatement) {
          return 'break;';
        } else if (s.type === AST_NODE_TYPES.ContinueStatement) {
          return 'continue;';
        } else if (s.type === AST_NODE_TYPES.WhileStatement) {
          return _this3.generateWhileStatement(s);
        } else if (s.type === AST_NODE_TYPES.DoWhileStatement) {
          return _this3.generateDoWhileStatement(s);
        } else {
          // tslint:disable-next-line:no-console
          console.warn("[WGSL compiler]: unknown statement: ".concat(stmt));
        }

        return '';
      }).join('\n');
    }
  }, {
    key: "generateReturnStatement",
    value: function generateReturnStatement(stmt) {
      // 必须匹配函数返回值类型，需要向上查找最近的的 FunctionDeclaration
      var returnType = traverseUpwards(stmt, function (currentNode) {
        if (currentNode.type === AST_NODE_TYPES.FunctionDeclaration && currentNode.returnType) {
          return currentNode.returnType;
        }
      });
      return "return ".concat(stmt.argument && this.generateExpression(stmt.argument, returnType) || '', ";");
    }
  }, {
    key: "generateWhileStatement",
    value: function generateWhileStatement(stmt) {
      // https://gpuweb.github.io/gpuweb/wgsl.html#loop-statement
      return "loop {\n  if (".concat(this.generateExpression(stmt.test), ") { break; }\n").concat(this.generateStatement(stmt.body), "\n}");
    }
  }, {
    key: "generateDoWhileStatement",
    value: function generateDoWhileStatement(stmt) {
      return "loop {\n".concat(this.generateStatement(stmt.body), "\n  if (").concat(this.generateExpression(stmt.test), ") { break; }\n}");
    }
  }, {
    key: "generateForStatement",
    value: function generateForStatement(node) {
      var _node$init, _node$init2;

      var init = '';

      if (((_node$init = node.init) === null || _node$init === void 0 ? void 0 : _node$init.type) === AST_NODE_TYPES.VariableDeclaration) {
        // 修改 init 类型例如 int i = 0;
        node.init.declarations.forEach(function (d) {
          d.id.dataType = AST_TOKEN_TYPES.Int32;
        });
        init = this.generateVariableDeclaration(node.init);
      } else if (((_node$init2 = node.init) === null || _node$init2 === void 0 ? void 0 : _node$init2.type) === AST_NODE_TYPES.AssignmentExpression) {
        init = this.generateExpression(node.init);
      }

      return "for (".concat(init, " ").concat(node.test && this.generateExpression(node.test), "; ").concat(node.update && this.generateExpression(node.update), ") {").concat(this.generateBlockStatement(node.body), "}");
    }
  }, {
    key: "generateIfStatement",
    value: function generateIfStatement(node) {
      var consequent = '';

      if (node.consequent.type === AST_NODE_TYPES.ExpressionStatement) {
        consequent = this.generateExpression(node.consequent.expression);
      } else if (node.consequent.type === AST_NODE_TYPES.BlockStatement) {
        consequent = this.generateBlockStatement(node.consequent);
      } else if (node.consequent.type === AST_NODE_TYPES.ReturnStatement) {
        consequent = this.generateReturnStatement(node.consequent);
      } else if (node.consequent.type === AST_NODE_TYPES.BreakStatement) {
        consequent = 'break;';
      }

      var alternate = '';

      if (node.alternate) {
        if (node.alternate.type === AST_NODE_TYPES.ExpressionStatement) {
          alternate = this.generateExpression(node.alternate.expression);
        } else if (node.alternate.type === AST_NODE_TYPES.BlockStatement) {
          alternate = this.generateBlockStatement(node.alternate);
        } else if (node.alternate.type === AST_NODE_TYPES.ReturnStatement) {
          alternate = this.generateReturnStatement(node.alternate);
        } else if (node.alternate.type === AST_NODE_TYPES.BreakStatement) {
          alternate = 'break;';
        }
      }

      return "if (".concat(this.generateExpression(node.test), ") {").concat(consequent, "}").concat(alternate ? "else {".concat(alternate, "}") : '');
    }
  }, {
    key: "generateFunctionExpression",
    value: function generateFunctionExpression(stmt) {
      var _this4 = this;

      // fn compute_main() -> void {
      if (stmt.body) {
        var _stmt$id, _stmt$id2;

        var attributes = '';
        var append = '';

        if (this.context && ((_stmt$id = stmt.id) === null || _stmt$id === void 0 ? void 0 : _stmt$id.name) === 'main') {
          var _this$context$threadG = _slicedToArray(this.context.threadGroupSize, 3),
              x = _this$context$threadG[0],
              y = _this$context$threadG[1],
              z = _this$context$threadG[2]; // @see https://github.com/gpuweb/gpuweb/pull/1046/files
          // attributes = `[[ workgroup_size(${x},${y},${z})]]\n`;

        } // every function must has a return statement


        var lastStatment = stmt.body.body[stmt.body.body.length - 1];

        if (lastStatment.type !== AST_NODE_TYPES.ReturnStatement) {
          append = '\nreturn;';
        }

        return "".concat(attributes, "fn ").concat((_stmt$id2 = stmt.id) === null || _stmt$id2 === void 0 ? void 0 : _stmt$id2.name, "(").concat((stmt.params || []).map(function (p) {
          return p.type === AST_NODE_TYPES.Identifier && "".concat(p.name, " : ").concat(_this4.generateDataType(p.dataType));
        }).join(', '), ") -> ").concat(this.generateDataType(stmt.returnType || AST_TOKEN_TYPES.Void), " {").concat(this.generateBlockStatement(stmt.body)).concat(append, "}");
      }

      return '';
    }
  }, {
    key: "generateExpression",
    value: function generateExpression(expression, dataType) {
      // @ts-ignore
      if (isScalar(expression === null || expression === void 0 ? void 0 : expression.type)) {
        return this.generateScalar( // @ts-ignore
        dataType || (expression === null || expression === void 0 ? void 0 : expression.type), // @ts-ignore
        expression.value); // @ts-ignore
      } else if (isVector(expression === null || expression === void 0 ? void 0 : expression.type)) {
        // @ts-ignore
        return this.generateVector( // @ts-ignore
        dataType || (expression === null || expression === void 0 ? void 0 : expression.type), // @ts-ignore
        expression.value); // @ts-ignore
      } else if (isMatrix(expression === null || expression === void 0 ? void 0 : expression.type)) {
        return this.generateMatrix( // @ts-ignore
        dataType || (expression === null || expression === void 0 ? void 0 : expression.type), // @ts-ignore
        expression.value);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === AST_NODE_TYPES.ArrayExpression) {
        return this.generateArrayExpression(expression, dataType);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === AST_NODE_TYPES.Identifier) {
        var _id$parent, _this$context;

        var id = getIdentifierFromScope(expression, expression.name);
        var storageClass = id === null || id === void 0 ? void 0 : (_id$parent = id.parent) === null || _id$parent === void 0 ? void 0 : _id$parent.storageClass; // 需要改变引用方式

        if (id && storageClass) {
          if (storageClass === STORAGE_CLASS.StorageBuffer) {
            var bufferIndex = this.context.uniforms.filter(function (u) {
              return u.storageClass === STORAGE_CLASS.StorageBuffer;
            }).findIndex(function (u) {
              return u.name === id.name;
            });

            if (bufferIndex > -1) {
              return "".concat(CodeGeneratorWGSL.GWEBGPU_BUFFER).concat(bufferIndex, ".").concat(expression.name);
            }
          } else if (storageClass === STORAGE_CLASS.Uniform) {
            var uniform = this.context.uniforms.filter(function (u) {
              return u.storageClass === STORAGE_CLASS.Uniform;
            }).find(function (u) {
              return u.name === id.name;
            });

            if (uniform) {
              return "".concat(CodeGeneratorWGSL.GWEBGPU_UNIFORM_PARAMS, ".").concat(expression.name);
            }
          }
        } // 如果引用内置函数


        if (componentWiseFunctions.indexOf(expression.name) > -1) {
          return "std::".concat(expression.name);
        } // 如果引用常量


        var define = (_this$context = this.context) === null || _this$context === void 0 ? void 0 : _this$context.defines.find(function (d) {
          return d.name === expression.name;
        });

        if (define) {
          var placeHolder = "".concat(DefineValuePlaceholder).concat(define === null || define === void 0 ? void 0 : define.name);
          return define.runtime ? placeHolder : "".concat(define.value);
        } // 如果使用内置类型转换函数 eg. i32(f32)
        // @see https://gpuweb.github.io/gpuweb/wgsl.html#conversion-expr


        if (typeCastFunctions.indexOf(expression.name) > -1 && (expression === null || expression === void 0 ? void 0 : expression.parent).type === AST_NODE_TYPES.CallExpression) {
          return this.generateDataType(transformDataTypeMap[expression.name]);
        }

        return this.castingDataType(dataType, expression.dataType, expression.name);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === AST_NODE_TYPES.BinaryExpression) {
        return this.generateBinaryExpression(expression, dataType);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === AST_NODE_TYPES.CallExpression) {
        return this.generateCallExpression(expression, dataType);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === AST_NODE_TYPES.AssignmentExpression) {
        return this.generateAssignmentExpression(expression, dataType);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === AST_NODE_TYPES.UpdateExpression) {
        return this.generateUpdateExpression(expression);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === AST_NODE_TYPES.MemberExpression) {
        return this.generateMemberExpression(expression);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === AST_NODE_TYPES.ConditionalExpression) {
        return this.generateConditionalExpression(expression);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === AST_NODE_TYPES.UnaryExpression) {
        return this.generateUnaryExpression(expression, dataType);
      } else {
        // tslint:disable-next-line:no-console
        console.warn("[WGSL compiler]: unknown expression: ".concat(expression), dataType);
      }

      return '';
    }
  }, {
    key: "generateUnaryExpression",
    value: function generateUnaryExpression(expression, dataType) {
      return "".concat(expression.operator).concat(this.generateExpression(expression.argument, dataType));
    }
  }, {
    key: "generateConditionalExpression",
    value: function generateConditionalExpression(expression) {
      // use select() instead of ternary operator
      // @see https://github.com/gpuweb/gpuweb/issues/826
      // @see https://gpuweb.github.io/gpuweb/wgsl.html#logical-builtin-functions
      return "select(".concat(this.generateExpression(expression.consequent), ", ").concat(this.generateExpression(expression.alternate), ", ").concat(this.generateExpression(expression.test), ")");
    }
  }, {
    key: "generateMemberExpression",
    value: function generateMemberExpression(expression) {
      var objectStr = this.generateExpression(expression.object);

      if (expression.property.type === AST_NODE_TYPES.Identifier) {
        if (!expression.computed) {
          // swizzling & struct uniform eg. params.u_k / vec.rgba
          return "".concat(objectStr, ".").concat(expression.property.name);
        } else {
          return "".concat(objectStr, "[").concat(expression.property.name, "]");
        }
      } else if ((expression.property.type === AST_TOKEN_TYPES.Float || expression.property.type === AST_TOKEN_TYPES.Int32 || expression.property.type === AST_TOKEN_TYPES.Uint32) && isFinite(Number(expression.property.value))) {
        var index = Number(expression.property.value);

        if (expression.object.type === AST_NODE_TYPES.Identifier && isArray(expression.object.dataType)) {
          // shared[0] 此时不能写成 shared.x
          return "".concat(objectStr, "[").concat(index, "]");
        } else {
          var swizzlingComponent = 'x';

          switch (index) {
            case 0:
              swizzlingComponent = 'x';
              break;

            case 1:
              swizzlingComponent = 'y';
              break;

            case 2:
              swizzlingComponent = 'z';
              break;

            case 3:
              swizzlingComponent = 'w';
              break;
          } // vec[0]
          // 考虑 getData()[0] 的情况，转译成 getData().x


          return "".concat(objectStr, ".").concat(swizzlingComponent);
        }
      } else {
        // data[a + b]
        return "".concat(objectStr, "[").concat(this.generateExpression(expression.property), "]");
      }
    }
  }, {
    key: "generateUpdateExpression",
    value: function generateUpdateExpression(expression) {
      var _ref = expression.argument,
          name = _ref.name; // don't support i++ or ++i
      // @see https://gpuweb.github.io/gpuweb/wgsl.html#for-statement

      if (expression.operator === '++') {
        return "".concat(name, " = ").concat(name, " + 1");
      } else if (expression.operator === '--') {
        return "".concat(name, " = ").concat(name, " - 1");
      }

      return expression.prefix ? "".concat(expression.operator).concat(name) : "".concat(name).concat(expression.operator);
    }
  }, {
    key: "generateAssignmentExpression",
    value: function generateAssignmentExpression(expression, dataType) {
      var left = this.generateExpression(expression.left, dataType);
      var right = this.generateExpression(expression.right, dataType); // wgsl don't support these operators: += *= -=...

      if (expression.operator.length > 1 && expression.operator.endsWith('=')) {
        var realOperator = expression.operator.substr(0, expression.operator.length - 1);
        return "".concat(left, " = ").concat(left, " ").concat(realOperator, " ").concat(right);
      }

      return "".concat(left, " ").concat(expression.operator, " ").concat(right);
    }
  }, {
    key: "generateArrayExpression",
    value: function generateArrayExpression(expression, dataType) {
      var _this5 = this;

      var dt = expression.dataType || dataType || AST_TOKEN_TYPES.Vector4Float; // vec3<f32>(1.0, 2.0, 3.0)

      return "".concat(this.generateDataType(dt), "(").concat(expression.elements.map(function (e) {
        return _this5.generateExpression(e, getComponentDataTypeFromVector(dt));
      }).join(', '), ")");
    }
  }, {
    key: "generateBinaryExpression",
    value: function generateBinaryExpression(expression, dataType) {
      var _expression$parent;

      var needBracket = // @ts-ignore
      ((_expression$parent = expression.parent) === null || _expression$parent === void 0 ? void 0 : _expression$parent.type) === AST_NODE_TYPES.BinaryExpression;
      return "".concat(needBracket ? '(' : '').concat(this.generateExpression(expression.left, dataType), " ").concat(expression.operator, " ").concat(this.generateExpression(expression.right, dataType)).concat(needBracket ? ')' : '');
    } // TODO: 替换 Math.sin() -> sin()

  }, {
    key: "generateCallExpression",
    value: function generateCallExpression(expression, dataType) {
      var _this6 = this;

      var isComponentWise = expression.callee.type === AST_NODE_TYPES.Identifier && componentWiseFunctions.indexOf(expression.callee.name) > -1;
      var params = [];

      if (expression.callee.type === AST_NODE_TYPES.Identifier) {
        // 获取函数声明时的参数类型
        var functionId = getIdentifierFromScope(expression, expression.callee.name, true);

        if (functionId && functionId.parent.type === AST_NODE_TYPES.FunctionDeclaration) {
          params = functionId.parent.params;
        }
      }

      return "".concat(this.generateExpression(expression.callee), "(").concat(expression.arguments.map(function (e, i) {
        // if (e.type === AST_NODE_TYPES.CallExpression) {
        //   // int(floor(v.x + 0.5))
        //   // 考虑函数嵌套的情况，需要丢弃掉外层推断的类型
        //   return this.generateExpression(e);
        // }
        return _this6.generateExpression(e, isComponentWise ? dataType : params[i] && params[i].dataType);
      }).join(', '), ")");
    }
  }, {
    key: "generateDataType",
    value: function generateDataType(dt) {
      return dataTypeMap[dt];
    }
  }, {
    key: "generateScalar",
    value: function generateScalar(dt, scalar) {
      if (dt === AST_TOKEN_TYPES.Boolean) {
        return "".concat(!!scalar);
      } else if (dt === AST_TOKEN_TYPES.Float) {
        return this.wrapFloat("".concat(Number(scalar)));
      } else if (dt === AST_TOKEN_TYPES.Int32 || dt === AST_TOKEN_TYPES.Uint32) {
        return parseInt("".concat(Number(scalar)), 10).toString();
      }

      return "".concat(scalar);
    }
  }, {
    key: "generateVector",
    value: function generateVector(dt, vector) {
      var componentDataType = getComponentDataTypeFromVector(dt);
      return this.castingDataType(dt, componentDataType, this.generateScalar(componentDataType, vector));
    }
  }, {
    key: "generateMatrix",
    value: function generateMatrix(dt, vector) {
      var componentDataType = getComponentDataTypeFromVector(dt);
      return this.castingDataType(dt, componentDataType, this.generateScalar(componentDataType, vector));
    }
  }, {
    key: "wrapFloat",
    value: function wrapFloat(float) {
      return float.indexOf('.') === -1 ? "".concat(float, ".0") : float;
    }
  }, {
    key: "castingDataType",
    value: function castingDataType(dt1, dt2, content) {
      // 需要强制类型转换
      if (dt1 && dt1 !== dt2) {
        var castDataType = compareDataTypePriority(dt1, dt2);

        if (castDataType !== dt2) {
          return "".concat(this.generateDataType(castDataType), "(").concat(content, ")");
        }
      }

      return content;
    }
  }, {
    key: "generateSwizzling",
    value: function generateSwizzling(dt) {
      var size = getComponentSize(dt);

      if (size === 3) {
        return '.rgb';
      } else if (size === 2) {
        return '.rg';
      } else if (size === 4) {
        return '.rgba';
      } else {
        return '.r';
      }
    }
  }, {
    key: "generateGlobalVariableDeclarations",
    value: function generateGlobalVariableDeclarations() {
      var _this7 = this;

      if (!this.context) {
        return '';
      }

      return "".concat(this.context.globalDeclarations.map(function (gd) {
        return "".concat(gd.shared && 'shared' || '', " ").concat(_this7.generateDataType(getComponentDataTypeFromVector(gd.type)).replace('[]', ''), " ").concat(gd.name, "[").concat(gd.value, "];");
      }).join('\n'), "\n");
    }
  }, {
    key: "generateUniforms",
    value: function generateUniforms() {
      var _this8 = this;

      if (!this.context) {
        return '';
      }

      var offset = 0;
      var uniformDeclarations = this.context.uniforms.map(function (uniform) {
        if (uniform.storageClass === STORAGE_CLASS.Uniform // WebGPU Compute Shader 使用 buffer 而非 uniform
        ) {
            var byteCount = getByteCount(uniform.type);
            var ret = "[[offset ".concat(offset, "]] ").concat(uniform.name, " : ").concat(_this8.generateDataType(uniform.type), ";");
            offset += byteCount;
            return ret;
          }

        return '';
      }).join('\n  ') // 缩进
      .trim();
      return uniformDeclarations && "type GWebGPUParams = [[block]] struct {\n  ".concat(uniformDeclarations, "\n};\n[[binding ").concat(++this.bufferBindingIndex, ", set 0]] var<uniform> ").concat(CodeGeneratorWGSL.GWEBGPU_UNIFORM_PARAMS, " : GWebGPUParams;");
    }
  }, {
    key: "generateBuffers",
    value: function generateBuffers() {
      var _this9 = this;

      if (!this.context) {
        return '';
      }

      var bufferIndex = -1;
      return this.context.uniforms.map(function (u) {
        if (u.storageClass === STORAGE_CLASS.StorageBuffer) {
          bufferIndex++;
          ++_this9.bufferBindingIndex;
          var componentDataType = getComponentDataTypeFromVector(u.type);
          return "type GWebGPUBuffer".concat(bufferIndex, " = [[block]] struct {\n  [[offset 0]] ").concat(u.name, " : [[stride ").concat(getByteCount(componentDataType), "]] array<").concat(_this9.generateDataType(componentDataType), ">;\n};\n[[binding ").concat(_this9.bufferBindingIndex, ", set 0]] var<storage_buffer> ").concat(CodeGeneratorWGSL.GWEBGPU_BUFFER).concat(bufferIndex, " : GWebGPUBuffer").concat(bufferIndex, ";"); //         } else if (u.type === 'image2D') {
          //           return `layout(set = 0, binding = ${++this
          //             .bufferBindingIndex}) uniform texture2D ${u.name};
          // layout(set = 0, binding = ${++this.bufferBindingIndex}) uniform sampler ${
          //             u.name
          //           }Sampler;
          // `;
        }

        return '';
      }).filter(function (line) {
        return line;
      }).join('\n').trim();
    }
  }]);

  return CodeGeneratorWGSL;
}();
CodeGeneratorWGSL.GWEBGPU_UNIFORM_PARAMS = 'gWebGPUUniformParams';
CodeGeneratorWGSL.GWEBGPU_BUFFER = 'gWebGPUBuffer';
//# sourceMappingURL=CodeGeneratorWGSL.js.map