import _defineProperty from "@babel/runtime/helpers/defineProperty";

var _typePriority;

import { AST_NODE_TYPES, AST_TOKEN_TYPES } from '../ast/glsl-ast-node-types';
import { traverseUpwards } from './node';
export function isLeft(node) {
  return !!traverseUpwards(node, function (currentNode) {
    return (// @ts-ignore
      (currentNode.type === AST_NODE_TYPES.BinaryExpression || // @ts-ignore
      currentNode.type === AST_NODE_TYPES.AssignmentExpression) && // @ts-ignore
      currentNode.left === node
    );
  });
}
export function isScalar(dt) {
  if (dt === AST_TOKEN_TYPES.Boolean || dt === AST_TOKEN_TYPES.Int32 || dt === AST_TOKEN_TYPES.Uint32 || dt === AST_TOKEN_TYPES.Float) {
    return true;
  }

  return false;
}
export function isVector(dt) {
  if (dt === AST_TOKEN_TYPES.Vector2Float || dt === AST_TOKEN_TYPES.Vector3Float || dt === AST_TOKEN_TYPES.Vector4Float || dt === AST_TOKEN_TYPES.Vector2Boolean || dt === AST_TOKEN_TYPES.Vector3Boolean || dt === AST_TOKEN_TYPES.Vector4Boolean || dt === AST_TOKEN_TYPES.Vector2Uint || dt === AST_TOKEN_TYPES.Vector3Uint || dt === AST_TOKEN_TYPES.Vector4Uint || dt === AST_TOKEN_TYPES.Vector2Int || dt === AST_TOKEN_TYPES.Vector3Int || dt === AST_TOKEN_TYPES.Vector4Int) {
    return true;
  }

  return false;
}
export function isMatrix(dt) {
  if (dt === AST_TOKEN_TYPES.Matrix3x3Float || dt === AST_TOKEN_TYPES.Matrix4x4Float) {
    return true;
  }

  return false;
}
export function isArray(dt) {
  if (dt === AST_TOKEN_TYPES.FloatArray || dt === AST_TOKEN_TYPES.Vector4FloatArray) {
    return true;
  }

  return false;
}
export function getComponentSize(dt) {
  if (dt === AST_TOKEN_TYPES.Vector2Boolean || dt === AST_TOKEN_TYPES.Vector2Float || dt === AST_TOKEN_TYPES.Vector2Int || dt === AST_TOKEN_TYPES.Vector2Uint) {
    return 2;
  } else if (dt === AST_TOKEN_TYPES.Vector3Boolean || dt === AST_TOKEN_TYPES.Vector3Float || dt === AST_TOKEN_TYPES.Vector3Int || dt === AST_TOKEN_TYPES.Vector3Uint) {
    return 3;
  } else if (dt === AST_TOKEN_TYPES.Vector4Boolean || dt === AST_TOKEN_TYPES.Vector4Float || dt === AST_TOKEN_TYPES.Vector4Int || dt === AST_TOKEN_TYPES.Vector4Uint) {
    return 4;
  }

  return 1;
}
export function getByteCount(dt) {
  if (dt === AST_TOKEN_TYPES.Vector2Boolean) {
    return 2;
  } else if (dt === AST_TOKEN_TYPES.Vector3Boolean) {
    return 3;
  } else if (dt === AST_TOKEN_TYPES.Vector4Boolean) {
    return 4;
  } else if (dt === AST_TOKEN_TYPES.Float || dt === AST_TOKEN_TYPES.Int32 || dt === AST_TOKEN_TYPES.Uint32) {
    return 4;
  } else if (dt === AST_TOKEN_TYPES.Vector2Float || dt === AST_TOKEN_TYPES.Vector2Int || dt === AST_TOKEN_TYPES.Vector2Uint) {
    return 8;
  } else if (dt === AST_TOKEN_TYPES.Vector3Float || dt === AST_TOKEN_TYPES.Vector3Int || dt === AST_TOKEN_TYPES.Vector3Uint) {
    return 12;
  } else if (dt === AST_TOKEN_TYPES.Vector4Float || dt === AST_TOKEN_TYPES.Vector4Int || dt === AST_TOKEN_TYPES.Vector4Uint) {
    return 16;
  } else if (dt === AST_TOKEN_TYPES.Matrix3x3Float) {
    return 36;
  } else if (dt === AST_TOKEN_TYPES.Matrix4x4Float) {
    return 64;
  }

  return 1;
}
/**
 * 从向量/矩阵数据类型中获取每个分量的数据类型
 * eg. vec2 -> float
 *     ivec2 -> int
 *     bvec4 -> bool
 *     mat3 -> float
 * vec4[] -> vec4
 * @param dt 向量类型
 */

export function getComponentDataTypeFromVector(dt) {
  if (dt === AST_TOKEN_TYPES.Vector2Boolean || dt === AST_TOKEN_TYPES.Vector3Boolean || dt === AST_TOKEN_TYPES.Vector4Boolean) {
    return AST_TOKEN_TYPES.Boolean;
  } else if (dt === AST_TOKEN_TYPES.Vector2Float || dt === AST_TOKEN_TYPES.Vector3Float || dt === AST_TOKEN_TYPES.Vector4Float || dt === AST_TOKEN_TYPES.FloatArray || dt === AST_TOKEN_TYPES.Matrix3x3Float || dt === AST_TOKEN_TYPES.Matrix4x4Float) {
    return AST_TOKEN_TYPES.Float;
  } else if (dt === AST_TOKEN_TYPES.Vector2Int || dt === AST_TOKEN_TYPES.Vector3Int || dt === AST_TOKEN_TYPES.Vector4Int) {
    return AST_TOKEN_TYPES.Int32;
  } else if (dt === AST_TOKEN_TYPES.Vector2Uint || dt === AST_TOKEN_TYPES.Vector3Uint || dt === AST_TOKEN_TYPES.Vector4Uint) {
    return AST_TOKEN_TYPES.Uint32;
  } else if (dt === AST_TOKEN_TYPES.Vector4FloatArray) {
    return AST_TOKEN_TYPES.Vector4Float;
  }

  return dt;
}
export var typePriority = (_typePriority = {}, _defineProperty(_typePriority, AST_TOKEN_TYPES.Void, 0), _defineProperty(_typePriority, AST_TOKEN_TYPES.Float, 1), _defineProperty(_typePriority, AST_TOKEN_TYPES.Int32, 2), _defineProperty(_typePriority, AST_TOKEN_TYPES.Uint32, 3), _defineProperty(_typePriority, AST_TOKEN_TYPES.Boolean, 4), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector2Float, 100), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector3Float, 101), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector4Float, 102), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector2Int, 200), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector3Int, 201), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector4Int, 202), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector2Uint, 300), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector3Uint, 301), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector4Uint, 302), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector2Boolean, 400), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector3Boolean, 401), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector4Boolean, 402), _defineProperty(_typePriority, AST_TOKEN_TYPES.Matrix3x3Float, 1000), _defineProperty(_typePriority, AST_TOKEN_TYPES.Matrix4x4Float, 1001), _defineProperty(_typePriority, AST_TOKEN_TYPES.FloatArray, 2000), _defineProperty(_typePriority, AST_TOKEN_TYPES.Vector4FloatArray, 2001), _typePriority);
/**
 * 用于强制类型转换
 * eg. float + int -> int(float) + int
 *     float + vec3 -> vec3(float) + vec3
 * @param type1 dt1
 * @param type2 dt2
 */

export function compareDataTypePriority(type1, type2) {
  if (typePriority[type1] && typePriority[type2]) {
    if (typePriority[type1] > typePriority[type2]) {
      return type1;
    } else {
      return type2;
    }
  }

  return type1;
}
//# sourceMappingURL=data-type.js.map