"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "DefineValuePlaceholder", {
  enumerable: true,
  get: function get() {
    return _gWebgpuCore.DefineValuePlaceholder;
  }
});
exports.Compiler = exports.FunctionPrependPlaceholder = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _gWebgpuCore = require("@antv/g-webgpu-core");

var _CodeGeneratorGLSL = require("./backends/CodeGeneratorGLSL100");

var _CodeGeneratorGLSL2 = require("./backends/CodeGeneratorGLSL450");

var _CodeGeneratorWGSL = require("./backends/CodeGeneratorWGSL");

var _ICodeGenerator = require("./backends/ICodeGenerator");

var _g = require("./pegjs/g");

var _Transformer = require("./Transformer");

// export interface Program {
//   type: 'Program';
//   body: Node[];
// }
var FunctionPrependPlaceholder = '__FunctionPrependPlaceholder__';
exports.FunctionPrependPlaceholder = FunctionPrependPlaceholder;

var Compiler = /*#__PURE__*/function () {
  function Compiler() {
    var _this$generators;

    (0, _classCallCheck2.default)(this, Compiler);
    this.target = _ICodeGenerator.Target.GLSL100;
    this.glslContext = {
      name: '',
      dispatch: [1, 1, 1],
      threadGroupSize: [1, 1, 1],
      maxIteration: 1,
      defines: [],
      uniforms: [],
      globalDeclarations: [],
      output: {
        name: ''
      },
      needPingpong: false
    };
    this.generators = (_this$generators = {}, (0, _defineProperty2.default)(_this$generators, _ICodeGenerator.Target.GLSL100, new _CodeGeneratorGLSL.CodeGeneratorGLSL100()), (0, _defineProperty2.default)(_this$generators, _ICodeGenerator.Target.GLSL450, new _CodeGeneratorGLSL2.CodeGeneratorGLSL450()), (0, _defineProperty2.default)(_this$generators, _ICodeGenerator.Target.WGSL, new _CodeGeneratorWGSL.CodeGeneratorWGSL()), _this$generators);
    this.transformer = new _Transformer.Transformer();
  }

  (0, _createClass2.default)(Compiler, [{
    key: "setTarget",
    value: function setTarget(target) {
      this.target = target;
      this.transformer.target = target;
      this.clear();
    }
  }, {
    key: "getContext",
    value: function getContext() {
      return this.glslContext;
    }
  }, {
    key: "getGenerator",
    value: function getGenerator() {
      return this.generators[this.target];
    }
    /**
     * 生成 AST
     */

  }, {
    key: "parse",
    value: function parse(source) {
      try {
        return (0, _g.parse)(source);
      } catch (e) {
        // tslint:disable-next-line:no-console
        console.error('[Parse error]:', e);
        throw e;
      }
    }
    /**
     * AST 变换
     * ESTree -> ShaderTree
     */

  }, {
    key: "transform",
    value: function transform(program) {
      return this.transformer.transform(program, this.glslContext);
    }
    /**
     * 代码生成
     */

  }, {
    key: "generateCode",
    value: function generateCode(program) {
      var generator = this.getGenerator();
      generator.clear();
      return generator.generate(program, this.glslContext);
    }
  }, {
    key: "clear",
    value: function clear() {
      this.glslContext = {
        name: '',
        dispatch: [1, 1, 1],
        threadGroupSize: [1, 1, 1],
        maxIteration: 1,
        defines: [],
        uniforms: [],
        globalDeclarations: [],
        output: {
          name: ''
        },
        needPingpong: false
      };
    }
  }, {
    key: "compileBundle",
    value: function compileBundle(gCode) {
      var _shaders,
          _this = this;

      var targets = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [_ICodeGenerator.Target.WGSL, _ICodeGenerator.Target.GLSL450, _ICodeGenerator.Target.GLSL100];
      var bundle = {
        shaders: (_shaders = {}, (0, _defineProperty2.default)(_shaders, _ICodeGenerator.Target.WGSL, ''), (0, _defineProperty2.default)(_shaders, _ICodeGenerator.Target.GLSL450, ''), (0, _defineProperty2.default)(_shaders, _ICodeGenerator.Target.GLSL100, ''), _shaders),
        context: undefined,
        toString: function toString() {
          return JSON.stringify(bundle).replace(/\\/g, '\\\\');
        }
      };
      targets.forEach(function (target) {
        _this.setTarget(target);

        var estree = _this.parse(gCode); // TODO: 应该只需要 transform 一次


        var shaderTree = _this.transform(estree);

        bundle.shaders[target] = _this.generateCode(shaderTree);
      }); // @ts-ignore

      bundle.context = this.getContext();
      return bundle;
    }
  }]);
  return Compiler;
}();

exports.Compiler = Compiler;
//# sourceMappingURL=Compiler.js.map