"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Transformer = exports.dataTypeMap = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _glslAstNodeTypes = require("./ast/glsl-ast-node-types");

var _tsAstNodeTypes = require("./ast/ts-ast-node-types");

var _ICodeGenerator = require("./backends/ICodeGenerator");

var _DataTypeInference = require("./DataTypeInference");

var _builtinFunctions = require("./utils/builtin-functions");

var _dataType = require("./utils/data-type");

var _flatten = _interopRequireDefault(require("./utils/flatten"));

var _node = require("./utils/node");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var PropertyDecorator;

(function (PropertyDecorator) {
  PropertyDecorator["In"] = "in";
  PropertyDecorator["Out"] = "out";
  PropertyDecorator["Shared"] = "shared";
})(PropertyDecorator || (PropertyDecorator = {}));

var dataTypeMap = {
  void: _glslAstNodeTypes.AST_TOKEN_TYPES.Void,
  bool: _glslAstNodeTypes.AST_TOKEN_TYPES.Boolean,
  int: _glslAstNodeTypes.AST_TOKEN_TYPES.Int32,
  uint: _glslAstNodeTypes.AST_TOKEN_TYPES.Uint32,
  float: _glslAstNodeTypes.AST_TOKEN_TYPES.Float,
  vec2: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Float,
  vec3: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Float,
  vec4: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Float,
  ivec2: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Int,
  ivec3: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int,
  ivec4: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Int,
  uvec2: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Uint,
  uvec3: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Uint,
  uvec4: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Uint,
  bvec2: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Boolean,
  bvec3: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Boolean,
  bvec4: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Boolean,
  mat3: _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix3x3Float,
  mat4: _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix4x4Float,
  'float[]': _glslAstNodeTypes.AST_TOKEN_TYPES.FloatArray,
  'vec4[]': _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4FloatArray
};
exports.dataTypeMap = dataTypeMap;
var emptyShaderStatement = {
  type: _glslAstNodeTypes.AST_NODE_TYPES.ReturnStatement,
  argument: null
};
/**
 * 将 ESTree 转换成 ShaderTree，同时完成 Scope 创建
 */

var Transformer = /*#__PURE__*/function () {
  function Transformer() {
    (0, _classCallCheck2.default)(this, Transformer);
    this.target = _ICodeGenerator.Target.GLSL100;
    this.inference = new _DataTypeInference.DataTypeInference();
    this.context = void 0;
  }

  (0, _createClass2.default)(Transformer, [{
    key: "transform",
    value: function transform(program, context) {
      var _this = this;

      if (context) {
        this.context = context;
      }

      var programNode = {
        type: _glslAstNodeTypes.AST_NODE_TYPES.Program,
        body: [],
        scope: [] // Root Scope

      }; // 添加内置变量

      this.addBuiltinVariablesToRootScope(programNode); // @ts-ignore

      programNode.body = (0, _flatten.default)(program.body.map(function (stmt) {
        return _this.transformStatement(stmt, programNode, programNode);
      })).filter(function (stmt) {
        return stmt;
      });
      return programNode;
    }
  }, {
    key: "addBuiltinVariablesToRootScope",
    value: function addBuiltinVariablesToRootScope(programNode) {
      (0, _node.addIdentifierToScope)(programNode, {
        type: _glslAstNodeTypes.AST_NODE_TYPES.Identifier,
        name: 'globalInvocationID',
        dataType: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int
      });
      (0, _node.addIdentifierToScope)(programNode, {
        type: _glslAstNodeTypes.AST_NODE_TYPES.Identifier,
        name: 'workGroupSize',
        dataType: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int
      });
      (0, _node.addIdentifierToScope)(programNode, {
        type: _glslAstNodeTypes.AST_NODE_TYPES.Identifier,
        name: 'workGroupID',
        dataType: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int
      });
      (0, _node.addIdentifierToScope)(programNode, {
        type: _glslAstNodeTypes.AST_NODE_TYPES.Identifier,
        name: 'localInvocationID',
        dataType: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int
      });
      (0, _node.addIdentifierToScope)(programNode, {
        type: _glslAstNodeTypes.AST_NODE_TYPES.Identifier,
        name: 'localInvocationIndex',
        dataType: _glslAstNodeTypes.AST_TOKEN_TYPES.Int32
      });
      (0, _node.addIdentifierToScope)(programNode, {
        type: _glslAstNodeTypes.AST_NODE_TYPES.Identifier,
        name: 'numWorkGroups',
        dataType: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int
      });
      (0, _node.addIdentifierToScope)(programNode, {
        type: _glslAstNodeTypes.AST_NODE_TYPES.Identifier,
        name: 'imageLoad',
        dataType: _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Float
      }, true);
    }
  }, {
    key: "transformStatement",
    value: function transformStatement(stmt, scopeNode, parentNode) {
      var _this2 = this;

      var shaderStatement = null;

      if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.VariableDeclaration) {
        shaderStatement = this.transformVariableDeclaration(stmt, scopeNode, parentNode);
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.FunctionDeclaration) {
        shaderStatement = this.transformFunctionDeclaration(stmt, parentNode);
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.DoWhileStatement) {
        shaderStatement = this.transformDoWhileStatement(stmt, parentNode);
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.WhileStatement) {
        shaderStatement = this.transformWhileStatement(stmt, parentNode);
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.IfStatement) {
        // eg. if (a > 10) {} else {}
        shaderStatement = this.transformIfStatement(stmt, parentNode);
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.ForStatement) {
        shaderStatement = this.transformForStatement(stmt, parentNode);
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.BreakStatement) {
        shaderStatement = {
          type: _glslAstNodeTypes.AST_NODE_TYPES.BreakStatement
        };
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.ContinueStatement) {
        shaderStatement = {
          type: _glslAstNodeTypes.AST_NODE_TYPES.ContinueStatement
        };
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.BlockStatement) {
        shaderStatement = {
          type: _glslAstNodeTypes.AST_NODE_TYPES.BlockStatement,
          parent: parentNode,
          scope: [],
          // Block Scope
          body: []
        }; // @ts-ignore

        shaderStatement.body = (0, _flatten.default)(stmt.body.map(function (s) {
          return _this2.transformStatement(s, shaderStatement, shaderStatement);
        }));
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.ReturnStatement) {
        var _parentNode$parent;

        shaderStatement = {
          type: _glslAstNodeTypes.AST_NODE_TYPES.ReturnStatement,
          parent: parentNode,
          argument: null
        }; // 需要找到 function 声明的返回类型 FunctionDeclaration

        shaderStatement.argument = this.transformExpression(stmt.argument, shaderStatement, ((_parentNode$parent = parentNode.parent) === null || _parentNode$parent === void 0 ? void 0 : _parentNode$parent.parent).returnType);
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.ExpressionStatement) {
        // @ts-ignore
        shaderStatement = {
          type: _glslAstNodeTypes.AST_NODE_TYPES.ExpressionStatement,
          parent: parentNode
        }; // @ts-ignore

        shaderStatement.expression = this.transformExpression(stmt.expression, shaderStatement);
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.ImportDeclaration) {
        // import { debug } from 'g-webgpu';
        stmt.specifiers.forEach(function (specifier) {
          if (specifier.type === _tsAstNodeTypes.AST_NODE_TYPES.ImportSpecifier) {
            // @ts-ignore
            var result = _builtinFunctions.exportFunctions[_this2.target][specifier.local.name];

            if (result) {
              // 引入内置函数/变量
              // TODO: 生成完整的 AST
              (0, _node.addIdentifierToScope)(scopeNode, {
                type: _glslAstNodeTypes.AST_NODE_TYPES.Identifier,
                name: specifier.local.name,
                dataType: result.returnType
              }, true);
              shaderStatement = {
                type: _glslAstNodeTypes.AST_NODE_TYPES.ImportedFunctionStatement,
                content: result.content
              };
            }
          }
        });
      } else if (stmt.type === _tsAstNodeTypes.AST_NODE_TYPES.ClassDeclaration) {
        shaderStatement = this.transformClassDeclaration(stmt, scopeNode, parentNode);
      }

      return shaderStatement;
    }
  }, {
    key: "transformExpression",
    value: function transformExpression(expression, parentNode, type) {
      var _this3 = this;

      var retExpression = null;

      if (expression) {
        if (expression.type === _tsAstNodeTypes.AST_NODE_TYPES.Identifier) {
          // 尝试在当前 Scope 中查找
          var existedIdentifier = (0, _node.getIdentifierFromScope)(parentNode, expression.name);
          retExpression = {
            type: _glslAstNodeTypes.AST_NODE_TYPES.Identifier,
            parent: parentNode,
            name: expression.name,
            dataType: existedIdentifier && existedIdentifier.dataType || type || _glslAstNodeTypes.AST_TOKEN_TYPES.Float
          };
        } else if (expression.type === _tsAstNodeTypes.AST_NODE_TYPES.Literal) {
          retExpression = {
            // @ts-ignore
            type: type || _glslAstNodeTypes.AST_TOKEN_TYPES.Float,
            parent: parentNode,
            value: expression.value
          };
        } else if (expression.type === _tsAstNodeTypes.AST_NODE_TYPES.AssignmentExpression) {
          // if (expression.operator === '*=') {
          //   console.log(expression.left, parentNode);
          //   debugger;
          // }
          // 右值类型由左侧推导
          var leftDataType = this.inference.infer(expression.left, parentNode);
          retExpression = {
            // @ts-ignore
            type: _glslAstNodeTypes.AST_NODE_TYPES.AssignmentExpression,
            parent: parentNode,
            // @ts-ignore
            left: null,
            // @ts-ignore
            right: null,
            operator: expression.operator
          }; // @ts-ignore

          retExpression.left = this.transformExpression(expression.left, // @ts-ignore
          retExpression, leftDataType); // @ts-ignore

          retExpression.right = this.transformExpression(expression.right, // @ts-ignore
          retExpression, leftDataType);
        } else if (expression.type === _tsAstNodeTypes.AST_NODE_TYPES.BinaryExpression || expression.type === _tsAstNodeTypes.AST_NODE_TYPES.LogicalExpression) {
          // 右值类型由左侧推导
          var leftRightDataType = type || this.inference.infer(expression, parentNode);
          retExpression = {
            type: _glslAstNodeTypes.AST_NODE_TYPES.BinaryExpression,
            parent: parentNode,
            // @ts-ignore
            left: null,
            // @ts-ignore
            right: null,
            operator: expression.operator
          }; // @ts-ignore

          retExpression.left = this.transformExpression(expression.left, // @ts-ignore
          retExpression, leftRightDataType); // @ts-ignore

          retExpression.right = this.transformExpression(expression.right, // @ts-ignore
          retExpression, leftRightDataType);
        } else if (expression.type === _tsAstNodeTypes.AST_NODE_TYPES.ArrayExpression) {
          retExpression = {
            type: _glslAstNodeTypes.AST_NODE_TYPES.ArrayExpression,
            parent: parentNode,
            elements: [],
            dataType: type
          }; // 需要根据 vec2<bool> -> bool

          var componentDataType = (0, _dataType.getComponentDataTypeFromVector)(type || _glslAstNodeTypes.AST_TOKEN_TYPES.Float);
          retExpression.elements = expression.elements.map(function (e) {
            return _this3.transformExpression(e, retExpression, componentDataType);
          });
        } else if (expression.type === _tsAstNodeTypes.AST_NODE_TYPES.UpdateExpression) {
          // @ts-ignore
          retExpression = {
            type: _glslAstNodeTypes.AST_NODE_TYPES.UpdateExpression,
            parent: parentNode,
            operator: expression.operator,
            // @ts-ignore
            argument: null,
            prefix: expression.prefix
          }; // @ts-ignore

          retExpression.argument = this.transformExpression(expression.argument, // @ts-ignore
          retExpression);
        } else if (expression.type === _tsAstNodeTypes.AST_NODE_TYPES.CallExpression) {
          retExpression = {
            type: _glslAstNodeTypes.AST_NODE_TYPES.CallExpression,
            parent: parentNode,
            // @ts-ignore
            callee: null,
            arguments: []
          }; // @ts-ignore

          retExpression.callee = this.transformExpression(expression.callee, // @ts-ignore
          retExpression); // @ts-ignore

          retExpression.arguments = expression.arguments.map(function (arg) {
            return (// @ts-ignore
              _this3.transformExpression(arg, retExpression)
            );
          });
        } else if (expression.type === _tsAstNodeTypes.AST_NODE_TYPES.MemberExpression) {
          // TODO: 在这里对 AST 进行改造，去掉 this.
          // eg. const a = this.sum(); -> const a = sum();
          if (expression.object.type === _tsAstNodeTypes.AST_NODE_TYPES.Identifier && expression.object.name === 'this') {
            return this.transformExpression(expression.property, parentNode);
          }

          retExpression = {
            type: _glslAstNodeTypes.AST_NODE_TYPES.MemberExpression,
            parent: parentNode,
            // @ts-ignore
            object: null,
            // @ts-ignore
            property: null,
            // @ts-ignore
            computed: expression.computed
          }; // @ts-ignore

          retExpression.object = this.transformExpression(expression.object, // @ts-ignore
          retExpression); // @ts-ignore

          retExpression.property = this.transformExpression(expression.property, // @ts-ignore
          retExpression);
          return retExpression;
        } else if (expression.type === _tsAstNodeTypes.AST_NODE_TYPES.ConditionalExpression) {
          retExpression = {
            type: _glslAstNodeTypes.AST_NODE_TYPES.ConditionalExpression,
            parent: parentNode,
            // @ts-ignore
            test: null,
            // @ts-ignore
            alternate: null,
            // @ts-ignore
            consequent: null
          }; // @ts-ignore

          retExpression.test = this.transformExpression(expression.test, // @ts-ignore
          retExpression); // @ts-ignore

          retExpression.alternate = this.transformExpression(expression.alternate, // @ts-ignore
          retExpression); // @ts-ignore

          retExpression.consequent = this.transformExpression(expression.consequent, // @ts-ignore
          retExpression);
        } else if (expression.type === _tsAstNodeTypes.AST_NODE_TYPES.UnaryExpression) {
          retExpression = {
            // @ts-ignore
            type: _glslAstNodeTypes.AST_NODE_TYPES.UnaryExpression,
            parent: parentNode,
            // @ts-ignore
            operator: expression.operator,
            prefix: expression.prefix
          }; // @ts-ignore

          retExpression.argument = this.transformExpression(expression.argument, // @ts-ignore
          retExpression);
        }
      }

      return retExpression;
    }
  }, {
    key: "transformIdentifier",
    value: function transformIdentifier(id, parent) {
      var name = id.name,
          typeAnnotation = id.typeAnnotation;
      return {
        type: _glslAstNodeTypes.AST_NODE_TYPES.Identifier,
        // TODO: 考虑 Struct
        dataType: typeAnnotation && dataTypeMap[typeAnnotation] || _glslAstNodeTypes.AST_TOKEN_TYPES.Float,
        name: name,
        parent: parent
      };
    }
  }, {
    key: "transformDoWhileStatement",
    value: function transformDoWhileStatement(stmt, parentNode) {
      var doWhileStatement = {
        type: _glslAstNodeTypes.AST_NODE_TYPES.DoWhileStatement,
        parent: parentNode,
        // @ts-ignore
        test: null,
        // @ts-ignore
        body: null
      }; // @ts-ignore

      doWhileStatement.test = this.transformExpression(stmt.test, doWhileStatement); // @ts-ignore

      doWhileStatement.body = this.transformStatement(stmt.body, doWhileStatement, doWhileStatement);
      return doWhileStatement;
    }
  }, {
    key: "transformWhileStatement",
    value: function transformWhileStatement(stmt, parentNode) {
      var whileStatement = {
        type: _glslAstNodeTypes.AST_NODE_TYPES.WhileStatement,
        parent: parentNode,
        // @ts-ignore
        test: null,
        // @ts-ignore
        body: null
      }; // @ts-ignore

      whileStatement.test = this.transformExpression(stmt.test, whileStatement); // @ts-ignore

      whileStatement.body = this.transformStatement(stmt.body, whileStatement, whileStatement);
      return whileStatement;
    }
  }, {
    key: "transformForStatement",
    value: function transformForStatement(stmt, parentNode) {
      var _stmt$init, _stmt$init2;

      var forStatement = {
        type: _glslAstNodeTypes.AST_NODE_TYPES.ForStatement,
        parent: parentNode,
        init: null,
        test: null,
        update: null,
        body: emptyShaderStatement
      };

      if (((_stmt$init = stmt.init) === null || _stmt$init === void 0 ? void 0 : _stmt$init.type) === _tsAstNodeTypes.AST_NODE_TYPES.VariableDeclaration) {
        // let i: int = 0;
        forStatement.init = this.transformStatement(stmt.init, parentNode, forStatement);
      } else if (((_stmt$init2 = stmt.init) === null || _stmt$init2 === void 0 ? void 0 : _stmt$init2.type) === _tsAstNodeTypes.AST_NODE_TYPES.AssignmentExpression) {
        // i = 0;
        forStatement.init = this.transformExpression(stmt.init, forStatement);
      }

      forStatement.test = this.transformExpression(stmt.test, forStatement, _glslAstNodeTypes.AST_TOKEN_TYPES.Int32 // 固定 i32 类型 eg. i < length
      );
      forStatement.update = this.transformExpression(stmt.update, forStatement);
      forStatement.body = this.transformStatement(stmt.body, parentNode, forStatement);
      return forStatement;
    }
  }, {
    key: "transformVariableDeclaration",
    value: function transformVariableDeclaration(stmt, scopeNode, parentNode) {
      var _this4 = this;

      // const a: int = 10;
      var shaderStatement = {
        type: _glslAstNodeTypes.AST_NODE_TYPES.VariableDeclaration,
        parent: parentNode,
        declarations: stmt.declarations.map(function (d) {
          var id = _this4.transformIdentifier(d.id); // 处理别名的情况，例如 const a = b; 此时 a 作为 b 的别名，当然 b 也有可能是别的变量的别名


          if (d.init && d.init.type === _tsAstNodeTypes.AST_NODE_TYPES.Identifier) {
            var scope = (0, _node.getScopeByIdentifierName)(scopeNode, d.init.name);

            if (scope) {
              scope.alias.push(d.init.name);
            }
          } // 加入变量 id 到 Scope 中


          (0, _node.addIdentifierToScope)(scopeNode, id, false); // 根据赋值语句推导 id 的类型

          if (!d.id.typeAnnotation) {
            var inferredDataType = _this4.inference.infer(d.init, scopeNode);

            id.dataType = inferredDataType;
          }

          var storageClass = _glslAstNodeTypes.STORAGE_CLASS.Private;
          /**
           * 全大写常量转译成 GLSL 常量
           * const AAA = 100;
           * ->
           * #define AAA 100
           */

          if (!scopeNode.parent && // 必须在 Root Scope 声明
          stmt.kind === 'const' && id.name === id.name.toUpperCase()) {
            var _d$init;

            var definedValue;
            var runtime = false;
            storageClass = _glslAstNodeTypes.STORAGE_CLASS.UniformConstant;

            if ((d === null || d === void 0 ? void 0 : (_d$init = d.init) === null || _d$init === void 0 ? void 0 : _d$init.type) === _tsAstNodeTypes.AST_NODE_TYPES.Literal && d.init.value !== null && isFinite(Number(d.init.value))) {
              // @ts-ignore
              definedValue = d.init.value;
            } else {
              runtime = true;
            }

            if (_this4.context) {
              _this4.context.defines.push({
                name: id.name,
                type: id.dataType,
                // @ts-ignore
                value: definedValue,
                runtime: runtime
              });
            }
          }

          var declarator = {
            type: _glslAstNodeTypes.AST_NODE_TYPES.VariableDeclarator,
            id: id,
            parent: shaderStatement,
            storageClass: storageClass,
            init: d.init && _this4.transformExpression(d.init, parentNode, id.dataType)
          };
          id.parent = declarator;
          return declarator;
        })
      };
      return shaderStatement;
    }
  }, {
    key: "transformFunctionDeclaration",
    value: function transformFunctionDeclaration(stmt, parentNode) {
      var _this5 = this;

      var id = this.transformIdentifier(stmt.id);
      var returnType = stmt.returnType && dataTypeMap[stmt.returnType] || _glslAstNodeTypes.AST_TOKEN_TYPES.Float;
      var shaderStatement = {
        type: _glslAstNodeTypes.AST_NODE_TYPES.FunctionDeclaration,
        parent: parentNode,
        id: id,
        body: {
          type: _glslAstNodeTypes.AST_NODE_TYPES.BlockStatement,
          body: [],
          scope: [] // Function Scope

        },
        params: [],
        returnType: returnType
      }; // 必须首先创建好父子关系，才能进行后续的递归创建

      id.parent = shaderStatement;
      shaderStatement.body.parent = shaderStatement; // 添加到函数名 Root Scope 中

      (0, _node.addIdentifierToScope)(parentNode, _objectSpread(_objectSpread({}, id), {}, {
        dataType: returnType
      }), true);
      shaderStatement.params = stmt.params.map(function (param) {
        var paramId = _this5.transformIdentifier(param, shaderStatement); // 添加参数到 Function Scope 中


        (0, _node.addIdentifierToScope)(shaderStatement.body, paramId, false);
        return paramId;
      }); // @ts-ignore

      shaderStatement.body.body = (0, _flatten.default)(stmt.body.body.map(function (b) {
        return _this5.transformStatement(b, shaderStatement, shaderStatement.body);
      }));
      return shaderStatement;
    }
  }, {
    key: "transformIfStatement",
    value: function transformIfStatement(stmt, parentNode) {
      var shaderStatement = {
        type: _glslAstNodeTypes.AST_NODE_TYPES.IfStatement,
        parent: parentNode,
        // @ts-ignore
        test: null,
        // @ts-ignore
        consequent: null,
        alternate: null
      };
      shaderStatement.test = this.transformExpression(stmt.test, shaderStatement);
      shaderStatement.consequent = this.transformStatement(stmt.consequent, shaderStatement, shaderStatement);

      if (stmt.alternate) {
        shaderStatement.alternate = this.transformStatement(stmt.alternate, shaderStatement, shaderStatement);
      }

      return shaderStatement; // 创建 Block Scope
    }
  }, {
    key: "transformClassDeclaration",
    value: function transformClassDeclaration(stmt, scopeNode, parentNode) {
      var _stmt$decorators,
          _this6 = this;

      var shaderStatement = [];
      (_stmt$decorators = stmt.decorators) === null || _stmt$decorators === void 0 ? void 0 : _stmt$decorators.forEach(function (decorator) {
        if (decorator.expression.type === _tsAstNodeTypes.AST_NODE_TYPES.CallExpression && decorator.expression.callee.type === _tsAstNodeTypes.AST_NODE_TYPES.Identifier && decorator.expression.callee.name === 'numthreads') {
          // numthreads(8, 1, 1)
          var threadGroupSize = decorator.expression.arguments.slice(0, 3).map(function (a) {
            return a.type === _tsAstNodeTypes.AST_NODE_TYPES.Literal && Number(a.value) || 1;
          });
          shaderStatement.push({
            type: _glslAstNodeTypes.AST_NODE_TYPES.NumThreadStatement,
            parent: parentNode,
            threadGroupSize: threadGroupSize
          });
          _this6.context.threadGroupSize = threadGroupSize;
        }
      });

      if (stmt.body.type === _tsAstNodeTypes.AST_NODE_TYPES.ClassBody) {
        /**
         * uniforms
         * eg. prop1: vec3[]
         */
        var classProperties = stmt.body.body.filter(function (e) {
          return e.type === _tsAstNodeTypes.AST_NODE_TYPES.ClassProperty;
        });

        if (classProperties.length) {
          var declaration = {
            type: _glslAstNodeTypes.AST_NODE_TYPES.VariableDeclaration,
            declarations: []
          };
          classProperties.forEach(function (property) {
            if (property.key.type === _tsAstNodeTypes.AST_NODE_TYPES.Identifier) {
              var id = _this6.transformIdentifier(property.key);

              (0, _node.addIdentifierToScope)(scopeNode, id, false);
              var storageClass = _glslAstNodeTypes.STORAGE_CLASS.Uniform; // 认为是 storage_buffer

              if (id.dataType === _glslAstNodeTypes.AST_TOKEN_TYPES.FloatArray || id.dataType === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4FloatArray) {
                storageClass = _glslAstNodeTypes.STORAGE_CLASS.StorageBuffer;
              }

              if (property.decorators && _this6.context) {
                _this6.analyzeDecorators(property.key.name, id.dataType, storageClass, property.decorators);
              } // eg. [[binding 0, set 0]] var<uniform> params : SimParams;
              //     [[binding 1, set 0]] var<storage_buffer> particlesA : Particles;


              var declarator = {
                type: _glslAstNodeTypes.AST_NODE_TYPES.VariableDeclarator,
                id: id,
                init: null,
                storageClass: storageClass
              };
              declaration.declarations.push(declarator); // @ts-ignore

              id.parent = declarator;
            }
          });
          shaderStatement.push(declaration);
        }

        var methodDefinitions = stmt.body.body.filter(function (e) {
          return e.type === _tsAstNodeTypes.AST_NODE_TYPES.MethodDefinition;
        });

        if (methodDefinitions.length) {
          methodDefinitions.forEach(function (method) {
            if (method.value.type === _tsAstNodeTypes.AST_NODE_TYPES.FunctionExpression) {
              // void main() 直接修改 AST
              if (method.decorators && method.decorators.length === 1) {
                method.value.returnType = 'void';
                method.value.params = [];
                method.value.id.name = 'main';
              }

              var functionDeclaration = _objectSpread(_objectSpread({}, method.value), {}, {
                type: _tsAstNodeTypes.AST_NODE_TYPES.FunctionDeclaration,
                body: method.value.body
              });

              shaderStatement.push(_this6.transformFunctionDeclaration(functionDeclaration, parentNode));
            }
          });
        }
      }

      return shaderStatement;
    }
  }, {
    key: "analyzeDecorators",
    value: function analyzeDecorators(propertyName, propertyType, storageClass, decorators) {
      var _this7 = this;

      var analyzeDecorators = decorators.map(function (d) {
        return _this7.extractDecorator(d);
      });

      if (analyzeDecorators.find(function (d) {
        return d.name === PropertyDecorator.Out;
      }) && analyzeDecorators.find(function (d) {
        return d.name === PropertyDecorator.In;
      })) {
        // 同时具备输入输出，此时需要通过 pingpong 实现
        this.context.needPingpong = true;
      }

      analyzeDecorators.forEach(function (_ref) {
        var name = _ref.name,
            params = _ref.params;

        if (name === PropertyDecorator.Shared) {
          _this7.context.globalDeclarations.push({
            name: propertyName,
            type: propertyType,
            shared: true,
            value: "".concat(params[0])
          });
        } else if (name === PropertyDecorator.Out || name === PropertyDecorator.In) {
          var existed = _this7.context.uniforms.find(function (u) {
            return u.name === propertyName;
          });

          if (!existed) {
            existed = {
              name: propertyName,
              type: propertyType,
              storageClass: storageClass,
              readonly: !!!analyzeDecorators.find(function (d) {
                return d.name === PropertyDecorator.Out;
              }),
              writeonly: !!!analyzeDecorators.find(function (d) {
                return d.name === PropertyDecorator.In;
              }),
              size: [Number(params[0]) || 1, Number(params[1]) || 1]
            };

            _this7.context.uniforms.push(existed);
          }

          if (name === PropertyDecorator.Out) {
            _this7.context.output.name = propertyName;
            _this7.context.output.size = existed.size;
            _this7.context.output.length = existed.size[0] * existed.size[1]; // if (propertyType === 'image2D') {
            //   this.context.output.typedArrayConstructor = Uint8ClampedArray;
            //   this.context.output.length! *= 4;
            // }
          }
        }
      });
    }
  }, {
    key: "extractDecorator",
    value: function extractDecorator(decorator) {
      var name = '';
      var params = [];

      if (decorator.expression.type === _tsAstNodeTypes.AST_NODE_TYPES.Identifier) {
        name = decorator.expression.name;
      } else if (decorator.expression.type === _tsAstNodeTypes.AST_NODE_TYPES.CallExpression && decorator.expression.callee.type === _tsAstNodeTypes.AST_NODE_TYPES.Identifier) {
        name = decorator.expression.callee.name;
        params = decorator.expression.arguments.map(function (a) {
          return a.type === _tsAstNodeTypes.AST_NODE_TYPES.Literal && Number(a.value) || 0;
        });
      }

      return {
        name: name,
        params: params
      };
    }
  }]);
  return Transformer;
}();

exports.Transformer = Transformer;
//# sourceMappingURL=Transformer.js.map