"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CodeGeneratorGLSL450 = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _gWebgpuCore = require("@antv/g-webgpu-core");

var _glslAstNodeTypes = require("../ast/glsl-ast-node-types");

var _builtinFunctions = require("../utils/builtin-functions");

var _dataType = require("../utils/data-type");

var _node = require("../utils/node");

var _ICodeGenerator = require("./ICodeGenerator");

var _dataTypeMap;

var dataTypeMap = (_dataTypeMap = {}, (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Void, 'void'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Boolean, 'bool'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Int32, 'int'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Uint32, 'uint'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Float, 'float'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Float, 'vec2'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Float, 'vec3'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Float, 'vec4'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Int, 'ivec2'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int, 'ivec3'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Int, 'ivec4'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Uint, 'uvec2'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Uint, 'uvec3'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Uint, 'uvec4'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Boolean, 'bvec2'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Boolean, 'bvec3'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Boolean, 'bvec4'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix3x3Float, 'mat3'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix4x4Float, 'mat4'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.FloatArray, 'sampler2D'), (0, _defineProperty2.default)(_dataTypeMap, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4FloatArray, 'sampler2D'), _dataTypeMap);

var CodeGeneratorGLSL450 = /*#__PURE__*/function () {
  function CodeGeneratorGLSL450() {
    (0, _classCallCheck2.default)(this, CodeGeneratorGLSL450);
    this.context = void 0;
    this.bufferBindingIndex = -1;
  }

  (0, _createClass2.default)(CodeGeneratorGLSL450, [{
    key: "clear",
    value: function clear() {
      this.bufferBindingIndex = -1;
    }
  }, {
    key: "generate",
    value: function generate(program, context) {
      var _this = this;

      if (context) {
        this.context = context;
      }

      var _ref = (context === null || context === void 0 ? void 0 : context.threadGroupSize) || [1, 1, 1],
          _ref2 = (0, _slicedToArray2.default)(_ref, 3),
          workGroupSizeX = _ref2[0],
          workGroupSizeY = _ref2[1],
          workGroupSizeZ = _ref2[2];

      var body = program.body.map(function (stmt) {
        return _this.generateStatement(stmt);
      }).join('\n');
      return "\n".concat(_builtinFunctions.importFunctions[_ICodeGenerator.Target.GLSL450], "\n\nbool gWebGPUDebug = false;\nvec4 gWebGPUDebugOutput = vec4(0.0);\n\nivec3 globalInvocationID = ivec3(gl_GlobalInvocationID);\nivec3 workGroupSize = ivec3(").concat(workGroupSizeX, ",").concat(workGroupSizeY, ",").concat(workGroupSizeZ, ");\nivec3 workGroupID = ivec3(gl_WorkGroupID);\nivec3 localInvocationID = ivec3(gl_LocalInvocationID);\nivec3 numWorkGroups = ivec3(gl_NumWorkGroups);\nint localInvocationIndex = int(gl_LocalInvocationIndex);\n\n").concat(this.generateUniforms(), "\n").concat(this.generateBuffers(), "\n").concat(this.generateGlobalVariableDeclarations(), "\n\n").concat(body, "\n");
    }
  }, {
    key: "generateStatement",
    value: function generateStatement(stmt) {
      if (stmt.type === _glslAstNodeTypes.AST_NODE_TYPES.VariableDeclaration) {
        return this.generateVariableDeclaration(stmt);
      } else if (stmt.type === _glslAstNodeTypes.AST_NODE_TYPES.FunctionDeclaration) {
        return this.generateFunctionExpression(stmt);
      } else if (stmt.type === _glslAstNodeTypes.AST_NODE_TYPES.BlockStatement) {
        return this.generateBlockStatement(stmt);
      } else if (stmt.type === _glslAstNodeTypes.AST_NODE_TYPES.NumThreadStatement) {
        return this.generateNumThreadStatement(stmt);
      } else if (stmt.type === _glslAstNodeTypes.AST_NODE_TYPES.ImportedFunctionStatement) {
        return stmt.content;
      } else {
        // tslint:disable-next-line:no-console
        console.warn("[GLSL450 compiler]: unknown statement: ".concat(stmt.type));
      }

      return '';
    }
  }, {
    key: "generateNumThreadStatement",
    value: function generateNumThreadStatement(stmt) {
      var _stmt$threadGroupSize = (0, _slicedToArray2.default)(stmt.threadGroupSize, 3),
          x = _stmt$threadGroupSize[0],
          y = _stmt$threadGroupSize[1],
          z = _stmt$threadGroupSize[2];

      return "layout (\n  local_size_x = ".concat(x, ",\n  local_size_y = ").concat(y, ",\n  local_size_z = ").concat(z, "\n) in;");
    }
  }, {
    key: "generateVariableDeclaration",
    value: function generateVariableDeclaration(stmt) {
      var _this2 = this;

      return stmt.declarations.map(function (declarator) {
        if (declarator.storageClass === _glslAstNodeTypes.STORAGE_CLASS.UniformConstant) {
          var _this2$context;

          var define = (_this2$context = _this2.context) === null || _this2$context === void 0 ? void 0 : _this2$context.defines.find(function (d) {
            return d.name === declarator.id.name;
          });
          var placeHolder = "".concat(_gWebgpuCore.DefineValuePlaceholder).concat(define === null || define === void 0 ? void 0 : define.name); // #define CONST 10

          return "#define ".concat(declarator.id.name, " ").concat((define === null || define === void 0 ? void 0 : define.runtime) ? placeHolder : _this2.generateExpression(declarator.init));
        } else if (declarator.storageClass === _glslAstNodeTypes.STORAGE_CLASS.Uniform || declarator.storageClass === _glslAstNodeTypes.STORAGE_CLASS.StorageBuffer) {// const name = declarator.id.name;
          // const componentDataType = getComponentDataTypeFromVector(
          //   declarator.id.dataType,
          // );
          // const componentDataTypeStr = this.generateDataType(componentDataType);
          // const dataType = this.generateDataType(declarator.id.dataType);
          // 不在这里生成
        } else if (declarator.storageClass === _glslAstNodeTypes.STORAGE_CLASS.Private) {
          if (declarator.init) {
            // float a = 1.0;
            // float b = vec3(1.0);
            return "".concat(_this2.generateDataType(declarator.id.dataType), " ").concat(declarator.id.name, " = ").concat(_this2.generateExpression(declarator.init, declarator.id.dataType), ";");
          } else {
            // float a;
            // Root Scope 中不允许声明变量但不赋值
            if (stmt.parent && stmt.parent.type === _glslAstNodeTypes.AST_NODE_TYPES.Program) {
              return '';
            }

            return "".concat(_this2.generateDataType(declarator.id.dataType), " ").concat(declarator.id.name, ";");
          }
        }

        return '';
      }).join('\n');
    }
  }, {
    key: "generateBlockStatement",
    value: function generateBlockStatement(stmt) {
      var _this3 = this;

      return stmt.body.map(function (s) {
        if (s.type === _glslAstNodeTypes.AST_NODE_TYPES.VariableDeclaration) {
          return _this3.generateVariableDeclaration(s);
        } else if (s.type === _glslAstNodeTypes.AST_NODE_TYPES.ExpressionStatement) {
          return _this3.generateExpression(s.expression) + ';';
        } else if (s.type === _glslAstNodeTypes.AST_NODE_TYPES.ReturnStatement) {
          return _this3.generateReturnStatement(s);
        } else if (s.type === _glslAstNodeTypes.AST_NODE_TYPES.IfStatement) {
          return _this3.generateIfStatement(s);
        } else if (s.type === _glslAstNodeTypes.AST_NODE_TYPES.ForStatement) {
          return _this3.generateForStatement(s);
        } else if (s.type === _glslAstNodeTypes.AST_NODE_TYPES.BreakStatement) {
          return 'break;';
        } else if (s.type === _glslAstNodeTypes.AST_NODE_TYPES.ContinueStatement) {
          return 'continue;';
        } else if (s.type === _glslAstNodeTypes.AST_NODE_TYPES.WhileStatement) {
          return _this3.generateWhileStatement(s);
        } else if (s.type === _glslAstNodeTypes.AST_NODE_TYPES.DoWhileStatement) {
          return _this3.generateDoWhileStatement(s);
        } else {
          // tslint:disable-next-line:no-console
          console.warn("[GLSL450 compiler]: unknown statement: ".concat(stmt));
        }

        return '';
      }).join('\n');
    }
  }, {
    key: "generateReturnStatement",
    value: function generateReturnStatement(stmt) {
      // 必须匹配函数返回值类型，需要向上查找最近的的 FunctionDeclaration
      var returnType = (0, _node.traverseUpwards)(stmt, function (currentNode) {
        if (currentNode.type === _glslAstNodeTypes.AST_NODE_TYPES.FunctionDeclaration && currentNode.returnType) {
          return currentNode.returnType;
        }
      });
      return "return ".concat(stmt.argument && this.generateExpression(stmt.argument, returnType) || '', ";");
    }
  }, {
    key: "generateWhileStatement",
    value: function generateWhileStatement(stmt) {
      return "while(".concat(this.generateExpression(stmt.test), ") {\n").concat(this.generateStatement(stmt.body), "\n}");
    }
  }, {
    key: "generateDoWhileStatement",
    value: function generateDoWhileStatement(stmt) {
      return "do {\n".concat(this.generateStatement(stmt.body), "\n} while(").concat(this.generateExpression(stmt.test), ");");
    }
  }, {
    key: "generateForStatement",
    value: function generateForStatement(node) {
      var _node$init, _node$init2;

      var init = '';

      if (((_node$init = node.init) === null || _node$init === void 0 ? void 0 : _node$init.type) === _glslAstNodeTypes.AST_NODE_TYPES.VariableDeclaration) {
        // 修改 init 类型例如 int i = 0;
        node.init.declarations.forEach(function (d) {
          d.id.dataType = _glslAstNodeTypes.AST_TOKEN_TYPES.Int32;
        });
        init = this.generateVariableDeclaration(node.init);
      } else if (((_node$init2 = node.init) === null || _node$init2 === void 0 ? void 0 : _node$init2.type) === _glslAstNodeTypes.AST_NODE_TYPES.AssignmentExpression) {
        init = this.generateExpression(node.init);
      }

      return "for (".concat(init, " ").concat(node.test && this.generateExpression(node.test), "; ").concat(node.update && this.generateExpression(node.update), ") {").concat(this.generateBlockStatement(node.body), "}");
    }
  }, {
    key: "generateIfStatement",
    value: function generateIfStatement(node) {
      var consequent = '';

      if (node.consequent.type === _glslAstNodeTypes.AST_NODE_TYPES.ExpressionStatement) {
        consequent = this.generateExpression(node.consequent.expression);
      } else if (node.consequent.type === _glslAstNodeTypes.AST_NODE_TYPES.BlockStatement) {
        consequent = this.generateBlockStatement(node.consequent);
      } else if (node.consequent.type === _glslAstNodeTypes.AST_NODE_TYPES.ReturnStatement) {
        consequent = this.generateReturnStatement(node.consequent);
      } else if (node.consequent.type === _glslAstNodeTypes.AST_NODE_TYPES.BreakStatement) {
        consequent = 'break;';
      }

      var alternate = '';

      if (node.alternate) {
        if (node.alternate.type === _glslAstNodeTypes.AST_NODE_TYPES.ExpressionStatement) {
          alternate = this.generateExpression(node.alternate.expression);
        } else if (node.alternate.type === _glslAstNodeTypes.AST_NODE_TYPES.BlockStatement) {
          alternate = this.generateBlockStatement(node.alternate);
        } else if (node.alternate.type === _glslAstNodeTypes.AST_NODE_TYPES.ReturnStatement) {
          alternate = this.generateReturnStatement(node.alternate);
        } else if (node.alternate.type === _glslAstNodeTypes.AST_NODE_TYPES.BreakStatement) {
          alternate = 'break;';
        }
      }

      return "if (".concat(this.generateExpression(node.test), ") {").concat(consequent, "}").concat(alternate ? "else {".concat(alternate, "}") : '');
    }
  }, {
    key: "generateFunctionExpression",
    value: function generateFunctionExpression(stmt) {
      var _this4 = this;

      if (stmt.body) {
        var _stmt$id;

        return "".concat(this.generateDataType(stmt.returnType || _glslAstNodeTypes.AST_TOKEN_TYPES.Void), " ").concat((_stmt$id = stmt.id) === null || _stmt$id === void 0 ? void 0 : _stmt$id.name, "(").concat((stmt.params || []).map(function (p) {
          return p.type === _glslAstNodeTypes.AST_NODE_TYPES.Identifier && "".concat(_this4.generateDataType(p.dataType), " ").concat(p.name);
        }).join(', '), ") {").concat(this.generateBlockStatement(stmt.body), "}");
      }

      return '';
    }
  }, {
    key: "generateExpression",
    value: function generateExpression(expression, dataType) {
      // @ts-ignore
      if ((0, _dataType.isScalar)(expression === null || expression === void 0 ? void 0 : expression.type)) {
        return this.generateScalar( // @ts-ignore
        dataType || (expression === null || expression === void 0 ? void 0 : expression.type), // @ts-ignore
        expression.value); // @ts-ignore
      } else if ((0, _dataType.isVector)(expression === null || expression === void 0 ? void 0 : expression.type)) {
        // @ts-ignore
        return this.generateVector( // @ts-ignore
        dataType || (expression === null || expression === void 0 ? void 0 : expression.type), // @ts-ignore
        expression.value); // @ts-ignore
      } else if ((0, _dataType.isMatrix)(expression === null || expression === void 0 ? void 0 : expression.type)) {
        return this.generateMatrix( // @ts-ignore
        dataType || (expression === null || expression === void 0 ? void 0 : expression.type), // @ts-ignore
        expression.value);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === _glslAstNodeTypes.AST_NODE_TYPES.ArrayExpression) {
        return this.generateArrayExpression(expression, dataType);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === _glslAstNodeTypes.AST_NODE_TYPES.Identifier) {
        var _id$parent;

        var id = (0, _node.getIdentifierFromScope)(expression, expression.name);
        var storageClass = id === null || id === void 0 ? void 0 : (_id$parent = id.parent) === null || _id$parent === void 0 ? void 0 : _id$parent.storageClass; // 需要改变引用方式

        if (id && storageClass) {
          if (storageClass === _glslAstNodeTypes.STORAGE_CLASS.StorageBuffer) {
            var bufferIndex = this.context.uniforms.filter(function (u) {
              return u.storageClass === _glslAstNodeTypes.STORAGE_CLASS.StorageBuffer;
            }).findIndex(function (u) {
              return u.name === id.name;
            });

            if (bufferIndex > -1) {
              return "".concat(CodeGeneratorGLSL450.GWEBGPU_BUFFER).concat(bufferIndex, ".").concat(expression.name);
            }
          } else if (storageClass === _glslAstNodeTypes.STORAGE_CLASS.Uniform) {
            var uniform = this.context.uniforms.filter(function (u) {
              return u.storageClass === _glslAstNodeTypes.STORAGE_CLASS.Uniform;
            }).find(function (u) {
              return u.name === id.name;
            });

            if (uniform) {
              return "".concat(CodeGeneratorGLSL450.GWEBGPU_UNIFORM_PARAMS, ".").concat(expression.name);
            }
          }
        }

        return this.castingDataType(dataType, expression.dataType, expression.name);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === _glslAstNodeTypes.AST_NODE_TYPES.BinaryExpression) {
        return this.generateBinaryExpression(expression, dataType);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === _glslAstNodeTypes.AST_NODE_TYPES.CallExpression) {
        return this.generateCallExpression(expression, dataType);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === _glslAstNodeTypes.AST_NODE_TYPES.AssignmentExpression) {
        return this.generateAssignmentExpression(expression, dataType);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === _glslAstNodeTypes.AST_NODE_TYPES.UpdateExpression) {
        return this.generateUpdateExpression(expression);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === _glslAstNodeTypes.AST_NODE_TYPES.MemberExpression) {
        return this.generateMemberExpression(expression);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === _glslAstNodeTypes.AST_NODE_TYPES.ConditionalExpression) {
        return this.generateConditionalExpression(expression);
      } else if ((expression === null || expression === void 0 ? void 0 : expression.type) === _glslAstNodeTypes.AST_NODE_TYPES.UnaryExpression) {
        return this.generateUnaryExpression(expression, dataType);
      } else {
        // tslint:disable-next-line:no-console
        console.warn("[GLSL450 compiler]: unknown expression: ".concat(expression), dataType);
      }

      return '';
    }
  }, {
    key: "generateUnaryExpression",
    value: function generateUnaryExpression(expression, dataType) {
      return "".concat(expression.operator).concat(this.generateExpression(expression.argument, dataType));
    }
  }, {
    key: "generateConditionalExpression",
    value: function generateConditionalExpression(expression) {
      // 条件判断也应该丢弃掉之前推断的类型
      return "(".concat(this.generateExpression(expression.test), ") ? (").concat(this.generateExpression(expression.consequent), ") : (").concat(this.generateExpression(expression.alternate), ")");
    }
  }, {
    key: "generateMemberExpression",
    value: function generateMemberExpression(expression) {
      var objectStr = this.generateExpression(expression.object);

      if (expression.property.type === _glslAstNodeTypes.AST_NODE_TYPES.Identifier) {
        if (!expression.computed) {
          // swizzling & struct uniform eg. params.u_k / vec.rgba
          return "".concat(objectStr, ".").concat(expression.property.name);
        } else {
          return "".concat(objectStr, "[").concat(expression.property.name, "]");
        }
      } else if ((expression.property.type === _glslAstNodeTypes.AST_TOKEN_TYPES.Float || expression.property.type === _glslAstNodeTypes.AST_TOKEN_TYPES.Int32 || expression.property.type === _glslAstNodeTypes.AST_TOKEN_TYPES.Uint32) && isFinite(Number(expression.property.value))) {
        var index = Number(expression.property.value);

        if (expression.object.type === _glslAstNodeTypes.AST_NODE_TYPES.Identifier && (0, _dataType.isArray)(expression.object.dataType)) {
          // shared[0] 此时不能写成 shared.x
          return "".concat(objectStr, "[").concat(index, "]");
        } else {
          var swizzlingComponent = 'x';

          switch (index) {
            case 0:
              swizzlingComponent = 'x';
              break;

            case 1:
              swizzlingComponent = 'y';
              break;

            case 2:
              swizzlingComponent = 'z';
              break;

            case 3:
              swizzlingComponent = 'w';
              break;
          } // vec[0]
          // 考虑 getData()[0] 的情况，转译成 getData().x


          return "".concat(objectStr, ".").concat(swizzlingComponent);
        }
      } else {
        // data[a + b]
        return "".concat(objectStr, "[").concat(this.generateExpression(expression.property), "]");
      }
    }
  }, {
    key: "generateUpdateExpression",
    value: function generateUpdateExpression(expression) {
      // i++ ++i
      return expression.prefix ? "".concat(expression.operator).concat(expression.argument.name) : "".concat(expression.argument.name).concat(expression.operator);
    }
  }, {
    key: "generateAssignmentExpression",
    value: function generateAssignmentExpression(expression, dataType) {
      if (expression.operator === '<<=') {
        console.log(expression, dataType);
      }

      var left = this.generateExpression(expression.left, dataType);
      var right = this.generateExpression(expression.right, dataType);
      return "".concat(left, " ").concat(expression.operator, " ").concat(right);
    }
  }, {
    key: "generateArrayExpression",
    value: function generateArrayExpression(expression, dataType) {
      var _this5 = this;

      var dt = expression.dataType || dataType || _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Float; // vec3(1.0, 2.0, 3.0)

      return "".concat(this.generateDataType(dt), "(").concat(expression.elements.map(function (e) {
        return _this5.generateExpression(e, (0, _dataType.getComponentDataTypeFromVector)(dt));
      }).join(', '), ")");
    }
  }, {
    key: "generateBinaryExpression",
    value: function generateBinaryExpression(expression, dataType) {
      var _expression$parent;

      var needBracket = // @ts-ignore
      ((_expression$parent = expression.parent) === null || _expression$parent === void 0 ? void 0 : _expression$parent.type) === _glslAstNodeTypes.AST_NODE_TYPES.BinaryExpression;
      return "".concat(needBracket ? '(' : '').concat(this.generateExpression(expression.left, dataType), " ").concat(expression.operator, " ").concat(this.generateExpression(expression.right, dataType)).concat(needBracket ? ')' : '');
    } // TODO: 替换 Math.sin() -> sin()

  }, {
    key: "generateCallExpression",
    value: function generateCallExpression(expression, dataType) {
      var _this6 = this;

      var isComponentWise = expression.callee.type === _glslAstNodeTypes.AST_NODE_TYPES.Identifier && _builtinFunctions.componentWiseFunctions.indexOf(expression.callee.name) > -1;
      var params = [];

      if (expression.callee.type === _glslAstNodeTypes.AST_NODE_TYPES.Identifier) {
        // 获取函数声明时的参数类型
        var functionId = (0, _node.getIdentifierFromScope)(expression, expression.callee.name, true);

        if (functionId && functionId.parent && functionId.parent.type === _glslAstNodeTypes.AST_NODE_TYPES.FunctionDeclaration) {
          params = functionId.parent.params;
        }
      }

      return "".concat(this.generateExpression(expression.callee), "(").concat(expression.arguments.map(function (e, i) {
        // if (e.type === AST_NODE_TYPES.CallExpression) {
        //   // int(floor(v.x + 0.5))
        //   // 考虑函数嵌套的情况，需要丢弃掉外层推断的类型
        //   return this.generateExpression(e);
        // }
        return _this6.generateExpression(e, isComponentWise ? dataType : params[i] && params[i].dataType);
      }).join(', '), ")");
    }
  }, {
    key: "generateDataType",
    value: function generateDataType(dt) {
      return dataTypeMap[dt];
    }
  }, {
    key: "generateScalar",
    value: function generateScalar(dt, scalar) {
      if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Boolean) {
        return "".concat(!!scalar);
      } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Float) {
        return this.wrapFloat("".concat(Number(scalar)));
      } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Int32 || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Uint32) {
        return parseInt("".concat(Number(scalar)), 10).toString();
      }

      return "".concat(scalar);
    }
  }, {
    key: "generateVector",
    value: function generateVector(dt, vector) {
      var componentDataType = (0, _dataType.getComponentDataTypeFromVector)(dt);
      return this.castingDataType(dt, componentDataType, this.generateScalar(componentDataType, vector));
    }
  }, {
    key: "generateMatrix",
    value: function generateMatrix(dt, vector) {
      var componentDataType = (0, _dataType.getComponentDataTypeFromVector)(dt);
      return this.castingDataType(dt, componentDataType, this.generateScalar(componentDataType, vector));
    }
  }, {
    key: "wrapFloat",
    value: function wrapFloat(float) {
      return float.indexOf('.') === -1 ? "".concat(float, ".0") : float;
    }
  }, {
    key: "castingDataType",
    value: function castingDataType(dt1, dt2, content) {
      // 需要强制类型转换
      if (dt1 && dt1 !== dt2) {
        var castDataType = (0, _dataType.compareDataTypePriority)(dt1, dt2);

        if (castDataType !== dt2) {
          return "".concat(this.generateDataType(castDataType), "(").concat(content, ")");
        }
      }

      return content;
    }
  }, {
    key: "generateSwizzling",
    value: function generateSwizzling(dt) {
      var size = (0, _dataType.getComponentSize)(dt);

      if (size === 3) {
        return '.rgb';
      } else if (size === 2) {
        return '.rg';
      } else if (size === 4) {
        return '.rgba';
      } else {
        return '.r';
      }
    }
  }, {
    key: "generateGlobalVariableDeclarations",
    value: function generateGlobalVariableDeclarations() {
      var _this7 = this;

      if (!this.context) {
        return '';
      }

      return "".concat(this.context.globalDeclarations.map(function (gd) {
        return "".concat(gd.shared && 'shared' || '', " ").concat(_this7.generateDataType((0, _dataType.getComponentDataTypeFromVector)(gd.type)).replace('[]', ''), " ").concat(gd.name, "[").concat(gd.value, "];");
      }).join('\n'), "\n");
    }
  }, {
    key: "generateUniforms",
    value: function generateUniforms() {
      var _this8 = this;

      if (!this.context) {
        return '';
      }

      var uniformDeclarations = this.context.uniforms.map(function (uniform) {
        if (uniform.storageClass === _glslAstNodeTypes.STORAGE_CLASS.Uniform // WebGPU Compute Shader 使用 buffer 而非 uniform
        ) {
            return "".concat(_this8.generateDataType(uniform.type), " ").concat(uniform.name, ";");
          }

        return '';
      }).join('\n  ') // 缩进
      .trim();
      return uniformDeclarations && "layout(std140, set = 0, binding = ".concat(++this.bufferBindingIndex, ") uniform GWebGPUParams {\n  ").concat(uniformDeclarations, "\n} ").concat(CodeGeneratorGLSL450.GWEBGPU_UNIFORM_PARAMS, ";");
    }
  }, {
    key: "generateBuffers",
    value: function generateBuffers() {
      var _this9 = this;

      if (!this.context) {
        return '';
      }

      var bufferIndex = -1;
      return this.context.uniforms.map(function (u) {
        if (u.storageClass === _glslAstNodeTypes.STORAGE_CLASS.StorageBuffer) {
          bufferIndex++;
          return "layout(std430, set = 0, binding = ".concat(++_this9.bufferBindingIndex, ") buffer ").concat(u.readonly && 'readonly' || '', " ").concat(u.writeonly && 'writeonly' || '', " GWebGPUBuffer").concat(bufferIndex, " {\n  ").concat(_this9.generateDataType((0, _dataType.getComponentDataTypeFromVector)(u.type)), " ").concat(u.name, "[];\n} ").concat(CodeGeneratorGLSL450.GWEBGPU_BUFFER).concat(bufferIndex, ";\n"); //         } else if (u.type === 'image2D') {
          //           return `layout(set = 0, binding = ${++this
          //             .bufferBindingIndex}) uniform texture2D ${u.name};
          // layout(set = 0, binding = ${++this.bufferBindingIndex}) uniform sampler ${
          //             u.name
          //           }Sampler;
          // `;
        }

        return '';
      }).filter(function (line) {
        return line;
      }).join('\n').trim();
    }
  }]);
  return CodeGeneratorGLSL450;
}();

exports.CodeGeneratorGLSL450 = CodeGeneratorGLSL450;
CodeGeneratorGLSL450.GWEBGPU_UNIFORM_PARAMS = 'gWebGPUUniformParams';
CodeGeneratorGLSL450.GWEBGPU_BUFFER = 'gWebGPUBuffer';
//# sourceMappingURL=CodeGeneratorGLSL450.js.map