"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.exportFunctions = exports.importFunctions = exports.typePriority = exports.swizzling = exports.componentWiseFunctions = exports.builtinFunctions = exports.typeCastFunctions = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _glslAstNodeTypes = require("../ast/glsl-ast-node-types");

var _ICodeGenerator = require("../backends/ICodeGenerator");

var _importFunctions, _exportFunctions;

// http://learnwebgl.brown37.net/12_shader_language/glsl_data_types.html
var typeCastFunctions = ['float', 'int', 'uint', 'vec2', 'vec3', 'vec4', 'bool', 'ivec2', 'ivec3', 'ivec4', 'bvec2', 'bvec3', 'bvec4', 'mat2', 'mat3', 'mat4'];
exports.typeCastFunctions = typeCastFunctions;
var builtinFunctions = ['radians', 'degrees', 'sin', 'cos', 'tan', 'asin', 'acos', 'atan', 'pow', 'exp', 'log', 'exp2', 'log2', 'sqrt', 'abs', 'sign', 'floor', 'ceil', 'min', 'max', 'normalize', 'distance']; // https://stackoverflow.com/questions/12085403/whats-the-logic-for-determining-a-min-max-vector-in-glsl
// https://www.khronos.org/registry/OpenGL/specs/gl/GLSLangSpec.4.30.pdf

exports.builtinFunctions = builtinFunctions;
var componentWiseFunctions = ['radians', 'degrees', 'sin', 'cos', 'tan', 'asin', 'acos', 'atan', 'pow', 'exp', 'log', 'exp2', 'log2', 'sqrt', 'abs', 'sign', 'floor', 'ceil', 'min', 'max'];
exports.componentWiseFunctions = componentWiseFunctions;
var swizzling = ['r', 'g', 'b', 'a', 'x', 'y', 'z', 'w', 's', 't', 'p', 'q'];
exports.swizzling = swizzling;
var typePriority = {
  float: 1,
  int: 2,
  uint: 3,
  vec2: 100,
  vec3: 101,
  vec4: 102
};
exports.typePriority = typePriority;
var importFunctions = (_importFunctions = {}, (0, _defineProperty2.default)(_importFunctions, _ICodeGenerator.Target.GLSL100, "\nfloat epsilon = 0.00001;\nvec2 addrTranslation_1Dto2D(float address1D, vec2 texSize) {\n  vec2 conv_const = vec2(1.0 / texSize.x, 1.0 / (texSize.x * texSize.y));\n  vec2 normAddr2D = float(address1D) * conv_const;\n  return vec2(fract(normAddr2D.x + epsilon), normAddr2D.y);\n}\n\nvoid barrier() {}\n  "), (0, _defineProperty2.default)(_importFunctions, _ICodeGenerator.Target.GLSL450, ''), (0, _defineProperty2.default)(_importFunctions, _ICodeGenerator.Target.WGSL, ''), _importFunctions);
exports.importFunctions = importFunctions;
var exportFunctions = (_exportFunctions = {}, (0, _defineProperty2.default)(_exportFunctions, _ICodeGenerator.Target.GLSL100, {
  debug: {
    content: "\nvoid debug(vec4 o) {\n  gWebGPUDebug = true;\n  gWebGPUDebugOutput = o;\n}\nvoid debug(vec3 o) {\n  debug(vec4(o.xyz, 0.0));\n}\nvoid debug(vec2 o) {\n  debug(vec4(o.xy, 0.0, 0.0));\n}\nvoid debug(ivec4 o) {\n  debug(vec4(o));\n}\nvoid debug(ivec3 o) {\n  debug(vec4(o.xyz, 0.0));\n}\nvoid debug(ivec2 o) {\n  debug(vec4(o.xy, 0.0, 0.0));\n}\nvoid debug(float o) {\n  debug(vec4(o, 0.0, 0.0, 0.0));\n}\nvoid debug(int o) {\n  debug(vec4(o, 0.0, 0.0, 0.0));\n}\nvoid debug(bool o) {\n  debug(vec4(o, 0.0, 0.0, 0.0));\n}\n",
    returnType: _glslAstNodeTypes.AST_TOKEN_TYPES.Void
  }
}), (0, _defineProperty2.default)(_exportFunctions, _ICodeGenerator.Target.GLSL450, {
  debug: {
    content: "\nvoid debug(vec4 o) {\n  gWebGPUDebug = true;\n  gWebGPUDebugOutput = o;\n}\nvoid debug(vec3 o) {\n  debug(vec4(o.xyz, 0.0));\n}\nvoid debug(vec2 o) {\n  debug(vec4(o.xy, 0.0, 0.0));\n}\nvoid debug(ivec4 o) {\n  debug(vec4(o));\n}\nvoid debug(ivec3 o) {\n  debug(vec4(o.xyz, 0.0));\n}\nvoid debug(ivec2 o) {\n  debug(vec4(o.xy, 0.0, 0.0));\n}\nvoid debug(float o) {\n  debug(vec4(o, 0.0, 0.0, 0.0));\n}\nvoid debug(int o) {\n  debug(vec4(o, 0.0, 0.0, 0.0));\n}\nvoid debug(bool o) {\n  debug(vec4(o, 0.0, 0.0, 0.0));\n}\n",
    returnType: _glslAstNodeTypes.AST_TOKEN_TYPES.Void
  }
}), (0, _defineProperty2.default)(_exportFunctions, _ICodeGenerator.Target.WGSL, {}), _exportFunctions);
exports.exportFunctions = exportFunctions;
//# sourceMappingURL=builtin-functions.js.map