"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isLeft = isLeft;
exports.isScalar = isScalar;
exports.isVector = isVector;
exports.isMatrix = isMatrix;
exports.isArray = isArray;
exports.getComponentSize = getComponentSize;
exports.getByteCount = getByteCount;
exports.getComponentDataTypeFromVector = getComponentDataTypeFromVector;
exports.compareDataTypePriority = compareDataTypePriority;
exports.typePriority = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _glslAstNodeTypes = require("../ast/glsl-ast-node-types");

var _node = require("./node");

var _typePriority;

function isLeft(node) {
  return !!(0, _node.traverseUpwards)(node, function (currentNode) {
    return (// @ts-ignore
      (currentNode.type === _glslAstNodeTypes.AST_NODE_TYPES.BinaryExpression || // @ts-ignore
      currentNode.type === _glslAstNodeTypes.AST_NODE_TYPES.AssignmentExpression) && // @ts-ignore
      currentNode.left === node
    );
  });
}

function isScalar(dt) {
  if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Boolean || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Int32 || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Uint32 || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Float) {
    return true;
  }

  return false;
}

function isVector(dt) {
  if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Boolean || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Boolean || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Boolean || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Uint || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Uint || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Uint || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Int || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Int) {
    return true;
  }

  return false;
}

function isMatrix(dt) {
  if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix3x3Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix4x4Float) {
    return true;
  }

  return false;
}

function isArray(dt) {
  if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.FloatArray || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4FloatArray) {
    return true;
  }

  return false;
}

function getComponentSize(dt) {
  if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Boolean || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Int || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Uint) {
    return 2;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Boolean || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Uint) {
    return 3;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Boolean || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Int || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Uint) {
    return 4;
  }

  return 1;
}

function getByteCount(dt) {
  if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Boolean) {
    return 2;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Boolean) {
    return 3;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Boolean) {
    return 4;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Int32 || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Uint32) {
    return 4;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Int || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Uint) {
    return 8;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Uint) {
    return 12;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Int || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Uint) {
    return 16;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix3x3Float) {
    return 36;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix4x4Float) {
    return 64;
  }

  return 1;
}
/**
 * 从向量/矩阵数据类型中获取每个分量的数据类型
 * eg. vec2 -> float
 *     ivec2 -> int
 *     bvec4 -> bool
 *     mat3 -> float
 * vec4[] -> vec4
 * @param dt 向量类型
 */


function getComponentDataTypeFromVector(dt) {
  if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Boolean || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Boolean || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Boolean) {
    return _glslAstNodeTypes.AST_TOKEN_TYPES.Boolean;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.FloatArray || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix3x3Float || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix4x4Float) {
    return _glslAstNodeTypes.AST_TOKEN_TYPES.Float;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Int || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Int) {
    return _glslAstNodeTypes.AST_TOKEN_TYPES.Int32;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Uint || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Uint || dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Uint) {
    return _glslAstNodeTypes.AST_TOKEN_TYPES.Uint32;
  } else if (dt === _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4FloatArray) {
    return _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Float;
  }

  return dt;
}

var typePriority = (_typePriority = {}, (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Void, 0), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Float, 1), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Int32, 2), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Uint32, 3), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Boolean, 4), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Float, 100), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Float, 101), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Float, 102), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Int, 200), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Int, 201), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Int, 202), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Uint, 300), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Uint, 301), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Uint, 302), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector2Boolean, 400), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector3Boolean, 401), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4Boolean, 402), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix3x3Float, 1000), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Matrix4x4Float, 1001), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.FloatArray, 2000), (0, _defineProperty2.default)(_typePriority, _glslAstNodeTypes.AST_TOKEN_TYPES.Vector4FloatArray, 2001), _typePriority);
/**
 * 用于强制类型转换
 * eg. float + int -> int(float) + int
 *     float + vec3 -> vec3(float) + vec3
 * @param type1 dt1
 * @param type2 dt2
 */

exports.typePriority = typePriority;

function compareDataTypePriority(type1, type2) {
  if (typePriority[type1] && typePriority[type2]) {
    if (typePriority[type1] > typePriority[type2]) {
      return type1;
    } else {
      return type2;
    }
  }

  return type1;
}
//# sourceMappingURL=data-type.js.map