"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addIdentifierToScope = addIdentifierToScope;
exports.getIdentifierFromScope = getIdentifierFromScope;
exports.getScopeByIdentifierName = getScopeByIdentifierName;
exports.traverseUpwards = traverseUpwards;

function addIdentifierToScope(node, id) {
  var isFunction = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  if (!node.scope) {
    node.scope = [];
  }

  if (!node.scope.find(function (v) {
    return v.id.name === id.name;
  })) {
    node.scope.push({
      id: id,
      alias: [],
      isFunction: isFunction
    });
  }
}
/**
 * 从当前节点向上查找，也要考虑别名
 * @param node 当前节点
 * @param idName id 名称
 */


function getIdentifierFromScope(node, idName) {
  var isFunction = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  return traverseUpwards(node, function (currentNode) {
    var _currentNode$scope;

    var existed = (_currentNode$scope = currentNode.scope) === null || _currentNode$scope === void 0 ? void 0 : _currentNode$scope.find(function (v) {
      return (v.id.name === idName || v.alias.includes(idName)) && (isFunction ? v.isFunction : true);
    });

    if (existed) {
      return existed.id;
    }
  });
}
/**
 * 从当前节点向上查找，也要考虑别名
 * @param node 当前节点
 * @param idName id 名称
 */


function getScopeByIdentifierName(node, idName) {
  var isFunction = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  return traverseUpwards(node, function (currentNode) {
    var _currentNode$scope2;

    var existed = (_currentNode$scope2 = currentNode.scope) === null || _currentNode$scope2 === void 0 ? void 0 : _currentNode$scope2.find(function (v) {
      return (v.id.name === idName || v.alias.includes(idName)) && (isFunction ? v.isFunction : true);
    });

    if (existed) {
      return existed;
    }
  });
}

function traverseUpwards(node, test) {
  var currentNode = node;

  while (currentNode) {
    var result = test(currentNode);

    if (result) {
      return result;
    }

    currentNode = currentNode.parent;
  }
}
//# sourceMappingURL=node.js.map